% program to generate figure for in-sample prediction exercise
% 10/02/2013
% Updates
% 10/07/2013: Tuning parameter selection is now in its own function.
%             New rotation scheme introduced for the penalty term
%             if opt.kill2007 applies, we standardize data again

%specify: taov, k
%Input: N,T sample size
%       taov: the vector of break points for trial
%       k: the largest number of factors we consider
clc; close all; clear all;

%% -------------------------------------------------------------------------
% SPECIFICATION
% - [Break date],[dataset],[kmax],[interval size],[rtc]
% -------------------------------------------------------------------------
doindfig = 1; %plot individual prediction series

% figure options
opt.r1 = 5;
opt.r2 = 6;

% other options
opt.ver  = 0; 
% 0: new lambda
% 1: old program but new lam
% 2: old program but new penalty
% 3: old program
opt.rtc  = 6;  %reciprocal of tuning constant
opt.kmax = 8; % maximum # factors
opt.d    = 21; 
% 1 : xdata_std, 
% 2 : xdata_uf_std   (unfiltered), 
% 11: NG's dataset  (same span as SW), unfiltered
% 12: NG's dataset  (longer span), unfiltered
% 21: SW version 03 (same span as SW), (bw = 100)
% 22: SW version 03 (same span as SW), unfiltered
% 23: SW version 03 (same span as SW), filtered (bw = 300)
% 31: SW version 03 (short), 
% 32: SW version 03 (short), unfiltered

opt.kill2007 = 0;

opt.bdate = '2007M05'; % break date
% opt.bdate = '2007M05'; % break date
% opt.bdate = '2007M03'; % break date
% opt.bdate = '2001M03';

opt.intsz  = 12;    % setting this to zero boils down to ``knowing break date'' situation.
intsz_l    = opt.intsz; % search interval size (lower)
intsz_u    = opt.intsz; % search interval size (upper)

% name of the specification
% - [Break date],[dataset],[kmax],[interval size], [rtc]
specname = ['Ver', num2str(opt.ver), '_b', opt.bdate, '_d', num2str(opt.d), 'k', num2str(opt.kmax), 'i', num2str(opt.intsz), 't', num2str(opt.rtc),'kill', num2str(opt.kill2007)];
tabname  = ['tab_', specname, '.xlsx']; %table name for xls file

%% -------------------------------------------------------------------------
% Path control
% -------------------------------------------------------------------------
% current folder
workpath = pwd;

% add toolbox
path_original     = path;
path_toolbox1  = genpath([workpath, '\', 'toolbox_mc']);
path_toolbox2  = genpath([workpath, '\', 'toolbox_pc']);
addpath(path_toolbox1); 
addpath(path_toolbox2);

% savepath
specname = [opt.bdate, '_d', num2str(opt.d), 'r', num2str(opt.r1), 'r', num2str(opt.r2)];
savepath = [workpath, '\result_pred\', specname];
chk_dir(savepath);

% datapath
datapath = [workpath, '\data'];


%% -------------------------------------------------------------------------
% Load DATA
% -------------------------------------------------------------------------
cd(datapath)
% data selection part 1
if opt.d <=10
    load('SW_xdata_ver02.mat'); % SW's data (old)
elseif (opt.d >10)&&(opt.d<=20)
    load('NG_xdata_ver02.mat'); % NG's data
elseif (opt.d == 21)||(opt.d == 22)
    load('SW_xdata_ver03_long.mat'); % SWver03 (long), bw=100
    elseif (opt.d == 23)
    load('SW_xdata_ver03_long_bw300.mat'); % SWver03 (long), bw=300
elseif (opt.d >30)&&(opt.d<=40) 
    load('SW_xdata_ver03_short.mat'); % SWver03 (short)
end
cd(workpath);

% data selection part 2
if (opt.d == 1)||(opt.d == 21)||(opt.d == 23)||(opt.d == 31) %filtered
    X = xdata_std'; %N by T form
elseif (opt.d == 2)||(opt.d == 22)||(opt.d == 32) %unfiltered
    X = xdata_uf_std'; %N by T form
elseif opt.d == 11
    X    = xdata_std(301:end,:)'; % N by T form, select 1985-2012 just as in SW
    xcal = xcal(301:end,:);
elseif opt.d == 12
    X = xdata_std(1:end,:)'; % N by T form, longer than SW, it starts from 1960
end


% if 2001M03 is pre-set break, we exclude post 2007 break data
if opt.kill2007 == 1
    if strcmp(opt.bdate,'2001M03')
        [X, xcal] = killpost2007(X,xcal);
         X=(X-repmat(mean(X,2),1,size(X,2)))./(repmat(std(X')',1,size(X,2)));
    end
end

[N,T] = size(X);


% catcode names
if (opt.d <=10)||(opt.d>20)
    xcatname = { ...
        'NIPA' ;...    %1'NIPA'
        'IP  ' ;...    %2'IP'
        'Emp ' ;...    %3'Emp/Unemp'
        'HSS ' ;...    %4'Hous.Starts'
        'Ord ' ;...    %5'Orders/Inv/Sales'
        'Pri ' ;...    %6'Prices'
        'Prod' ;...    %7'EmpEarning/Productivity'
        'IntL' ;...    %8'Int. Rates' Level
        'IntS' ;...    %9'Int. Rates' Spread
        'Mon ' ;...    %10'Money&Credit'
        'StPr' ;...    %11'StockPrices&Wealth'
        'HoPr' ;...    %12'Housing Prices'
        'ExR ' ;...    %13'Exch.Rates'
        '' ;...        %14'' not used
        'Others';...   %15'Other': 1) consumer expectation, 2) political uncertainty
        };
    % Int Rate: Level and spread
    xcatcode(xcatcode>9,1) = xcatcode(xcatcode>9,1) + 1;
    xcatcode(floor(xcatcode)==8,1) = xcatcode(floor(xcatcode)==8,1) + 1;
    xcatcode(xcatcode == 9.01,1) = 8.01;
    xcatcode(xcatcode == 9.02,1) = 8.02;
    % tab_xname = [num2cell((1:1:size(xname,1))') xname (xcatname(floor(xcatcode))) num2cell(xcatcode)];
       
elseif (opt.d >10)&&(opt.d<20)
    xcatname = { ...
        'Output ' ;...    %1'output'
        'Labor  ' ;...    %2'Labor'
        'Housing' ;...    %3'Housing'
        'Ord    ' ;...    %4'Orders/Inv/Sales'
        'Money  ' ;...    %5'Money&Credit'
        'Bond   ' ;...    %6'Bond'
        'Pri    ' ;...    %7'Prices'
        'StPr   ' ;...    %8'Stock market'
        'ExR    ' ;...    %9'Exchange rate'
        };
end
    
%% -------------------------------------------------------------------------
% Preliminary Setting
% -------------------------------------------------------------------------
% % Penalization tuning constant (default is 1 for both)
% Fconst1 = 1/opt.rtc; %on Ftuning1
% Fconst2 = 1/opt.rtc; %on Ftuning2

% Back out T0 (conjectured break point)
temp_y    = str2double(opt.bdate(1:4)); %year
temp_m    = str2double(opt.bdate(6:7)); %month
temp_date = temp_y + (temp_m-1)/12;
temp_indx = (1:1:size(xcal,1))';
T0        = temp_indx(xcal == temp_date,1);

% Break region setting
% tao_0 = 272/T; % center of the break region to check
tao_0 = T0/T; % center of the break region to check
step  = 1/T;   % step size for window (1/T means monthly frequency step)

%region to search break dates
tao_l   = tao_0-step*intsz_l; %lower bound for the break region
tao_u   = tao_0+step*intsz_u; %upper bound for the break region
taov    = tao_l:step:tao_u; %vector of the break region
taovcal = xcal(int16(taov*T)'); %break region calendar

% maximum number of factors
k = opt.kmax;


%% Factor estimation at the estimated break date
T1_hat = floor(T*tao_0);
T2_hat = T-T1_hat;

X1 = X(:,1:T1_hat);
X2 = X(:,T1_hat+1:end);

[F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1_hat,T2_hat,k);


F1  = F1_til(:,1:opt.r1); 
L1  = lambda1_til(:,1:opt.r1);

F2  = F2_til(:,1:opt.r2); 
L2  = lambda2_til(:,1:opt.r2);


% case1: Using old factor (meaning we use old loading to span F)
% F2o = X2'*L1;
% L2o = (F2o'*F2o)\(F2o'*X2');
% P1  = F2o*L2o;

% F2o = (L1'*L1)\(L1'*X2);
% P1  = F2o'*L1';

% case2: Rotate factors/loadings
R   = maxcol(L2,L1);
F2r = F2*R(:,1:opt.r1);
L2r = L2*R(:,1:opt.r1);
P2  = F2r*L2r';

% case3: With additional factor
if opt.r1 < opt.r2
    F2  = F2_til(:,1:opt.r2);
    L2  = lambda2_til(:,1:opt.r2);
    P3  = F2*L2';
else
    P3 = P2;
end
% case1: Old loading, we use rotated factor here
P1 = (F2r*L1'); %post-break

P10 = F1*L1';


%% Figure - individual series
nnn = 36;
xcal00 = xcal(T1_hat-nnn:T1_hat);
xcal01 = xcal(T1_hat+1:end);

% path
indfigpath = [savepath, '\individual'];
chk_dir(indfigpath);
cd(indfigpath);

if doindfig == 1
% draw figures
for varind = 1:1:N;
    
    fig = figure(1111);
    setmyfig(fig, [1.7, 1.2, 12, 8]);
    
    % Some series
    temp_XX   = [X1(varind,end-nnn:end)'; X2(varind,:)']; %actual
    temp_P1   = [P10(end-nnn:end,varind); P1(:,varind)]; % case 1
    temp_xcal = [xcal00; xcal01];
    
    % figure
    plot(temp_xcal, temp_XX, 'color', rgb('silver'), 'linewidth', 4);
    hold on
    %     plot(xcal01, P1(:,varind), 'k--', 'linewidth', 4);
    plot(xcal01, P3(:,varind), 'b', 'linewidth', 5);
    plot(xcal01, P2(:,varind), 'r--', 'linewidth', 5);
    plot(temp_xcal, temp_P1, 'k--', 'linewidth', 3);
    %plot(xcal01, X2(varind,:)', 'color', rgb('silver'), 'linewidth', 4);

    
    aaa = axis;
    plot([xcal(T1_hat), xcal(T1_hat)], [aaa(3), aaa(4)], 'r--', 'linewidth', 5);
    
    
    hold off
    title([num2str(varind), ' : ', xname{varind}], 'fontsize', 30);
    
    %     l = legend({'Old L, Old F', 'New L, Old F', 'New L, New F', 'Actual'});
    %     set(l, 'fontsize', 20);
    set(gca, 'fontsize', 30, 'linewidth', 3);
    
    % save 
    savefigname = ['fig_v',num2str(varind), '.png'];
    saveas(fig, savefigname);
end
end % doindfig 

% variable list
tab_xname = [num2cell((1:1:size(xname,1))') xname (xcatname(floor(xcatcode))) num2cell(xcatcode)];
xlswrite('tab_xname.xls', tab_xname);
cd(workpath);

%% Calculate R2

e1 = X2'-P1;
e2 = X2'-P2;
e3 = X2'-P3;

R2 = zeros(N,3);
adjR2 = zeros(N,3);

for varind = 1:1:N
    
    R2(varind,1) = 1 - e1(:,varind)'*e1(:,varind)/( X2(varind,:)*X2(varind,:)' );
    R2(varind,2) = 1 - e2(:,varind)'*e2(:,varind)/( X2(varind,:)*X2(varind,:)' );
    R2(varind,3) = 1 - e3(:,varind)'*e3(:,varind)/( X2(varind,:)*X2(varind,:)' );
    
    nnn = size(X2(varind,:),2);
    ppp1 = opt.r1; ppp2 = opt.r1; ppp3 = opt.r2;
    adjR2(varind,1) = 1 - (1-R2(varind,1))*(nnn-1)/(nnn-ppp1-1);
    adjR2(varind,2) = 1 - (1-R2(varind,2))*(nnn-1)/(nnn-ppp2-1);
    adjR2(varind,3) = 1 - (1-R2(varind,3))*(nnn-1)/(nnn-ppp3-1);
    
    
end


% plot(adjR2, 'linewidth', 2);
% plot(R2, 'linewidth', 2);


disp('# of series that ...');
disp(['R3 > R1 :', num2str(sum(adjR2(:,3)>adjR2(:,1))), ' out of ', num2str(N)]);
disp(['R3 > R2 :', num2str(sum(adjR2(:,3)>adjR2(:,2))), ' out of ', num2str(N)]);


% Save table
tab_head = {'', 'Name', 'case1', 'case2', 'case3', 'c2-c1', 'c3-c2'};
tab00 = [num2cell((1:1:N)'), xname, num2cell(adjR2), num2cell(adjR2(:,2)-adjR2(:,1)) num2cell(adjR2(:,3)-adjR2(:,2))];
tab01 = [num2cell((1:1:N)'), xname, num2cell(R2), num2cell(R2(:,2)-R2(:,1)) num2cell(R2(:,3)-R2(:,2))];

tab  = [tab_head; tab00];
tab1 = [tab_head; tab01];

cd(savepath);
xlswrite('tab_adjR2.xls', tab, 'adjR2');
xlswrite('tab_adjR2.xls', tab1, 'R2');
cd(workpath);



%% R2 figure - Adjusted R2
% generate xtick based on xcatcode
xcatname00 = xcatname;
% xcatname00{10} = 'Mo';
% xcatname00{11} = 'Sp';
xcatname00{15} = '';

xindx = (1:1:size(xname,1))';
cindx = zeros(size(xname,1),1);

fcatcode = floor(xcatcode);
ucatcode = unique(fcatcode);
nunqcode = length(unique(fcatcode));

for jj = 1:1:nunqcode
    temp_indx = xindx(fcatcode == ucatcode(jj));
    
    cindx(temp_indx(end),1) = ucatcode(jj);
end
xtick      = xindx(cindx~=0);
xticklabel = xcatname00(cindx(cindx~=0));


% fig
fig = figure(1);
setmyfig(fig);
bar(adjR2(:,2)-adjR2(:,1), 'basevalue', 0, 'facecolor', rgb('lightgrey'));
hold on
plot([1,N], [0, 0], 'r--', 'linewidth', 3)
hold off
xlim([-2, N+4])
% title('adj. R2: Case 2 over Case 1', 'fontsize', 25);
set(gca, 'xtick', xtick, 'xticklabel', xticklabel, 'fontsize', 20, 'linewidth', 3);
set(gca, 'xgrid', 'on', 'gridlinestyle', '-')
rotateXLabels( gca(), 45 )
ylim([-0.05,1]);
cd(savepath);
saveas(fig, ['fig_adjR2_case21_r', num2str(opt.r1), 'r', num2str(opt.r2), 'b', opt.bdate, '.png']);
cd(workpath);

fig = figure(2);
setmyfig(fig);
bar(adjR2(:,3)-adjR2(:,1),'basevalue', 0, 'facecolor', rgb('lightgrey'));
hold on
plot([1,N], [0, 0], 'r--', 'linewidth', 3)
hold off
xlim([-2, N+4])
% title('adj. R2: Case 3 over Case 1', 'fontsize', 25);
set(gca, 'xtick', xtick, 'xticklabel', xticklabel, 'fontsize', 20, 'linewidth', 3);
set(gca, 'xgrid', 'on', 'gridlinestyle', '-')
rotateXLabels( gca(), 45 )
ylim([-0.05,1]);
cd(savepath);
saveas(fig, ['fig_adjR2_case31_r', num2str(opt.r1), 'r', num2str(opt.r2), 'b', opt.bdate,'.png']);
cd(workpath);

fig = figure(3);
setmyfig(fig);
% plot(adjR2(:,3)-adjR2(:,2), 'linewidth', 3);
bar(adjR2(:,3)-adjR2(:,2), 'basevalue', 0, 'facecolor', rgb('lightgrey'));
hold on
plot([1,N], [0, 0], 'r--', 'linewidth', 3)
hold off
xlim([-2, N+4])
% title('adj. R2: Case 3 over Case 2', 'fontsize', 25);
set(gca, 'xtick', xtick, 'xticklabel', xticklabel, 'fontsize', 20, 'linewidth', 3);
set(gca, 'xgrid', 'on', 'gridlinestyle', '-')
rotateXLabels( gca(), 45 )
ylim([-0.05,1]);
cd(savepath);
saveas(fig, ['fig_adjR2_case32_r', num2str(opt.r1), 'r', num2str(opt.r2), 'b', opt.bdate,'.png']);
cd(workpath);


%% R2 fig - raw R2

fig = figure(1);
setmyfig(fig);
bar(R2(:,2)-R2(:,1), 'basevalue', 0, 'facecolor', rgb('lightgrey'));
hold on
plot([1,N], [0, 0], 'r--', 'linewidth', 3)
hold off
xlim([-2, N+4])
% title('R2: Case 2 over Case 1', 'fontsize', 25);
set(gca, 'xtick', xtick, 'xticklabel', xticklabel, 'fontsize', 20, 'linewidth', 3);
set(gca, 'xgrid', 'on', 'gridlinestyle', '-')
rotateXLabels( gca(), 45 )
ylim([-0.03,1]);
cd(savepath);
saveas(fig, ['fig_R2_case21_r', num2str(opt.r1), 'r', num2str(opt.r2), 'b', opt.bdate,'.png']);
cd(workpath);

fig = figure(2);
setmyfig(fig);
bar(R2(:,3)-R2(:,1),'basevalue', 0, 'facecolor', rgb('lightgrey'));
hold on
plot([1,N], [0, 0], 'r--', 'linewidth', 3)
hold off
xlim([-2, N+4])
% title('R2: Case 3 over Case 1', 'fontsize', 25);
set(gca, 'xtick', xtick, 'xticklabel', xticklabel, 'fontsize', 20, 'linewidth', 3);
set(gca, 'xgrid', 'on', 'gridlinestyle', '-')
rotateXLabels( gca(), 45 )
ylim([-0.03,1]);
cd(savepath);
saveas(fig, ['fig_R2_case31_r', num2str(opt.r1), 'r', num2str(opt.r2), 'b', opt.bdate,'.png']);
cd(workpath);

fig = figure(3);
setmyfig(fig);
% plot(adjR2(:,3)-adjR2(:,2), 'linewidth', 3);
bar(R2(:,3)-R2(:,2), 'basevalue', 0, 'facecolor', rgb('lightgrey'));
hold on
plot([1,N], [0, 0], 'r--', 'linewidth', 3)
hold off
xlim([-2, N+4])
% title('adj. R2: Case 3 over Case 2', 'fontsize', 25);
set(gca, 'xtick', xtick, 'xticklabel', xticklabel, 'fontsize', 20, 'linewidth', 3);
set(gca, 'xgrid', 'on', 'gridlinestyle', '-')
rotateXLabels( gca(), 45 )
ylim([-0.03,1]);
% ylim([0, 1]);
cd(savepath);
saveas(fig, ['fig_R2_case32_r', num2str(opt.r1), 'r', num2str(opt.r2), 'b', opt.bdate,'.png']);
cd(workpath);

%% R2-together
fig = figure(4);
setmyfig(fig);
% plot(adjR2(:,3)-adjR2(:,2), 'linewidth', 3);
bar(R2(:,3)-R2(:,2), 'basevalue', 0, 'facecolor', rgb('midnightblue'));
hold on
bar(-(R2(:,2)-R2(:,1)), 'basevalue', 0, 'facecolor', rgb('silver'));
plot([1,N], [0, 0], 'r--', 'linewidth', 3)
hold off
xlim([-2, N+4])
% title('R2: Case 3 over Case 2', 'fontsize', 25);
set(gca, 'xtick', xtick, 'xticklabel', xticklabel, 'fontsize', 15, 'linewidth', 2);
set(gca, 'xgrid', 'on', 'gridlinestyle', '-')
set(gca, 'linewidth', 3);
rotateXLabels( gca(), 45 )
ylim([-0.8,0.6]);
% grid on
cd(savepath);
saveas(fig, ['fig_R2_all_r', num2str(opt.r1), 'r', num2str(opt.r2), 'b', opt.bdate,'.png']);
cd(workpath);


%% R2 not difference

fig = figure(7);
setmyfig(fig);
% plot(adjR2(:,3)-adjR2(:,2), 'linewidth', 3);
bar(R2(:,1), 'basevalue', 0, 'facecolor', rgb('silver'));
hold on
plot([1,N], [0, 0], 'r--', 'linewidth', 3)
hold off
xlim([-2, N+4])
% title('R2: Case 3 over Case 2', 'fontsize', 25);
set(gca, 'xtick', xtick, 'xticklabel', xticklabel, 'fontsize', 20, 'linewidth', 2);
set(gca, 'xgrid', 'on', 'gridlinestyle', '-')
set(gca, 'linewidth', 3);
rotateXLabels( gca(), 45 )
% ylim([-1,1]);
% grid on
cd(savepath);
saveas(fig, ['fig_R2_case1_r', num2str(opt.r1), 'r', num2str(opt.r2), 'b', opt.bdate,'.png']);
cd(workpath);

fig = figure(8);
setmyfig(fig);
% plot(adjR2(:,3)-adjR2(:,2), 'linewidth', 3);
bar(R2(:,2), 'basevalue', 0, 'facecolor', rgb('silver'));
hold on
plot([1,N], [0, 0], 'r--', 'linewidth', 3)
hold off
xlim([-2, N+4])
% title('R2: Case 3 over Case 2', 'fontsize', 25);
set(gca, 'xtick', xtick, 'xticklabel', xticklabel, 'fontsize', 20, 'linewidth', 2);
set(gca, 'xgrid', 'on', 'gridlinestyle', '-')
set(gca, 'linewidth', 3);
rotateXLabels( gca(), 45 )
ylim([-0.03,1]);
% grid on
cd(savepath);
saveas(fig, ['fig_R2_case2_r', num2str(opt.r1), 'r', num2str(opt.r2), 'b', opt.bdate,'.png']);
cd(workpath);

fig = figure(9);
setmyfig(fig);
% plot(adjR2(:,3)-adjR2(:,2), 'linewidth', 3);
bar(R2(:,3), 'basevalue', 0, 'facecolor', rgb('silver'));
hold on
plot([1,N], [0, 0], 'r--', 'linewidth', 3)
hold off
xlim([-2, N+4])
% title('R2: Case 3 over Case 2', 'fontsize', 25);
set(gca, 'xtick', xtick, 'xticklabel', xticklabel, 'fontsize', 20, 'linewidth', 2);
set(gca, 'xgrid', 'on', 'gridlinestyle', '-')
set(gca, 'linewidth', 3);
rotateXLabels( gca(), 45 )
ylim([-0.03,1]);
% grid on
cd(savepath);
saveas(fig, ['fig_R2_case3_r', num2str(opt.r1), 'r', num2str(opt.r2), 'b', opt.bdate,'.png']);
cd(workpath);













