%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  This Program Runs Empirical Application of Group Lasso Estimation of Factor Models.
% Part2: Factor Identification
%                  This version: July 17th, 2013
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% updates
% 7/22/2013: More options to the dataset (opt.d=1 is our default)
% 10/07/2013: Tuning parameter selection is now in its own function.
%             New rotation scheme introduced for the penalty term
%             if opt.kill2007 applies, we standardize data again

% preliminary
clc; close all; clear all;

%% SOME INPUTS
opt.d    = 21; 
% 1 : xdata_std, 
% 2 : xdata_uf_std   (unfiltered), 
% 11: NG's dataset  (same span as SW), unfiltered
% 12: NG's dataset  (longer span), unfiltered
% 21: SW version 03 (same span as SW), (bw = 100)
% 22: SW version 03 (same span as SW), unfiltered
% 23: SW version 03 (same span as SW), filtered (bw = 300)
% 31: SW version 03 (short), 
% 32: SW version 03 (short), unfiltered

opt.kill2007 = 0;
bdate = '2007M05'; %break date
% bdate = '2001M03';

r1 = 5;
r2 = 6;

%% housekeeping
% Path spec
workpath = pwd;
datapath = [workpath, '\data'];

% file names 
specname = ['B', bdate, 'd', num2str(opt.d), 'R', num2str(r1), 'R', num2str(r2), 'kill', num2str(opt.kill2007)];
tabname  = ['tab_part2_', specname, '.xlsx'];

% save path
savepath = [workpath, '\result_part2\', specname];
chk_dir(savepath);


%% -------------------------------------------------------------------------
% Load DATA
% -------------------------------------------------------------------------
cd(datapath);
% data selection part 1
if opt.d <=10
    load('SW_xdata_ver02.mat'); % SW's data (old)
elseif (opt.d >10)&&(opt.d<=20)
    load('NG_xdata_ver02.mat'); % NG's data
elseif (opt.d == 21)||(opt.d == 22)
    load('SW_xdata_ver03_long.mat'); % SWver03 (long), bw=100
    elseif (opt.d == 23)
    load('SW_xdata_ver03_long_bw300.mat'); % SWver03 (long), bw=300
elseif (opt.d >30)&&(opt.d<=40) 
    load('SW_xdata_ver03_short.mat'); % SWver03 (short)
end

% data selection part 2
if (opt.d == 1)||(opt.d == 21)||(opt.d == 23)||(opt.d == 31) %filtered
    X = xdata_std'; %N by T form
elseif (opt.d == 2)||(opt.d == 22)||(opt.d == 32) %unfiltered
    X = xdata_uf_std'; %N by T form
elseif opt.d == 11
    X    = xdata_std(301:end,:)'; % N by T form, select 1985-2012 just as in SW
    xcal = xcal(301:end,:);
elseif opt.d == 12
    X = xdata_std(1:end,:)'; % N by T form, longer than SW, it starts from 1960
end

% if 2001M03 is pre-set break, we exclude post 2007 break data
if opt.kill2007 == 1
    if strcmp(bdate,'2001M03')
        [X, xcal] = killpost2007(X,xcal);
        X=(X-repmat(mean(X,2),1,size(X,2)))./(repmat(std(X')',1,size(X,2)));
    end
end

[N,T] = size(X);

% catcode names
if (opt.d <=10)||(opt.d>20)
    xcatname = { ...
        'NIPA' ;...    %1'NIPA'
        'IP  ' ;...    %2'IP'
        'Emp ' ;...    %3'Emp/Unemp'
        'HSS ' ;...    %4'Hous.Starts'
        'Ord ' ;...    %5'Orders/Inv/Sales'
        'Pri ' ;...    %6'Prices'
        'Prod' ;...    %7'EmpEarning/Productivity'
        'IntL' ;...    %8'Int. Rates' Level
        'IntS' ;...    %9'Int. Rates' Spread
        'Mon ' ;...    %10'Money&Credit'
        'StPr' ;...    %11'StockPrices&Wealth'
        'HoPr' ;...    %12'Housing Prices'
        'ExR ' ;...    %13'Exch.Rates'
        '' ;...        %14'' not used
        'Others';...   %15'Other': 1) consumer expectation, 2) political uncertainty
        };
    % Int Rate: Level and spread
    xcatcode(xcatcode>9,1) = xcatcode(xcatcode>9,1) + 1;
    xcatcode(floor(xcatcode)==8,1) = xcatcode(floor(xcatcode)==8,1) + 1;
    xcatcode(xcatcode == 9.01,1) = 8.01;
    xcatcode(xcatcode == 9.02,1) = 8.02;
       
elseif (opt.d >10)&&(opt.d<20)
    xcatname = { ...
        'Output ' ;...    %1'output'
        'Labor  ' ;...    %2'Labor'
        'Housing' ;...    %3'Housing'
        'Ord    ' ;...    %4'Orders/Inv/Sales'
        'Money  ' ;...    %5'Money&Credit'
        'Bond   ' ;...    %6'Bond'
        'Pri    ' ;...    %7'Prices'
        'StPr   ' ;...    %8'Stock market'
        'ExR    ' ;...    %9'Exchange rate'
        };
end


% size
[N,T] = size(X);

% Back out T0 (break point)
temp_y    = str2double(bdate(1:4)); %year
temp_m    = str2double(bdate(6:7)); %month
temp_date = temp_y + (temp_m-1)/12;
temp_indx = (1:1:size(xcal,1))';
T0        = temp_indx(xcal == temp_date,1);

T1 = T0;  %276: 2007Dec
T2 = T-T1;

cd(workpath);


%% step1: preliminary
% - sort data in xcatcode order
[sXcatcode, Ix] = sort(xcatcode, 'ascend');
sX = X(Ix,:);

fXcatcode = floor(sXcatcode); % floor
Jset = unique(fXcatcode);     % unique categories
J    = length(Jset); % # of unique category number

if J<max(r1,r2)
    disp('max(r1,r2) is larger than # of categories');
end

Jfreq = histc(fXcatcode, Jset); % frequency of catcode

disp('-------------------------------');
disp('# of individual series in each category');
disp([xcatname(Jset,1), num2cell(Jfreq)]);

%% step2: Factor estimation
X1 = sX(:,1:T1); %N*T1
X2 = sX(:,T1+1:T1+T2); %N*T2

% full sample
[f0,eig,f00] = svd(sX'*sX);
F_til        = f0(:,1:r2)*sqrt(T); %T*r2
lambda_til   = sX*F_til/T; %N*r1

% Pre-break
[f0,eig,f00] = svd(X1'*X1);
F1_til       = f0(:,1:r1)*sqrt(T1); %T1*r1
lambda1_til  = X1*F1_til/T1; %N*r1

% Post-break
[f0,eig,f00] = svd(X2'*X2);
F2_til       = f0(:,1:r2)*sqrt(T2); %T2*r2
lambda2_til  = X2*F2_til/T2; %N*r2


%% step3: Factor identification (rotation) - Pre-break sample
temp_R2  = zeros(J,1);
temp_bet = zeros(r1, J);

% loop over categories
for j=1:1:J
    jcat = Jset(j);
    
    xi  = X1(fXcatcode == jcat,:); % category j's data
    nxi = size(xi,1);
    
    % get the first pc for cat j
    [f0,eig,f00] = svd(xi'*xi);
    fxi          = f0(:,1)*sqrt(T1);
    lxi          = xi*fxi/T1;
    
    % OLS (without intercept)
    [bet, R2] = myols(fxi, F1_til);
    
    % store
    temp_bet(:,j) = bet;
    temp_R2(j)    = R2;
    
end

% Select factor rotation based on the largest R2
[sR2, Ix]  = sort(temp_R2, 'descend');
sbet       = temp_bet(:,Ix);
Omega1_til = sbet(1:r1,1:r1); % factor rotation for the pre-break sample

sJset    = Jset(Ix); % sort Jset as well
r2Jset   = sJset(r1+1:end); % kill cat that were used to construct F1
selJ     = sJset(1:r1); %selected categories

% Rotated factors and loadings
RF1_til      = F1_til*Omega1_til; % T1*r1
Rlambda1_til = (Omega1_til\lambda1_til')'; %n*r1

% First sample factor explanation order:
Ix1  = Ix;
sR21 = sR2;
% xcatname(Ix1)

%% Factor identification (rotation) - Post-break sample
% step 1: X2' = Rlambda1_til*F12_til + resid
% - Extending the rotated pre-break factor into the post-break period
F12_til = myols(X2, Rlambda1_til)';

% step 2: F12_til = F2*C0 + resid
C0 = myols(F12_til, F2_til);

% step 3 (only when r2>r1): We can use the variables groups j that were not
% used to construct RF1_til.

nadd     = min(r2-r1,J-r1); %# of additional factors
if nadd > 0
    
    temp_R2  = zeros(length(r2Jset),1);
    temp_bet = zeros(r2, length(r2Jset));
    
    for j = 1:1:length(r2Jset)
        jcat = r2Jset(j);
        
        xi  = X2(fXcatcode == jcat,:); % category j's data
        nxi = size(xi,1);
        
        % get the first pc for cat j
        [f0,eig,f00] = svd(xi'*xi);
        fxi          = f0(:,1)*sqrt(T2);
        lxi          = xi*fxi/T2;
        
        % OLS (without intercept)
        [bet, R2] = myols(fxi, F2_til);
        
        % store
        temp_bet(:,j) = bet;
        temp_R2(j)    = R2;
        
    end
    
    % Select factor rotation based on the largest R2
    [sR2, Ix]  = sort(temp_R2, 'descend');
    sbet       = temp_bet(:,Ix);
    Cadd       = sbet(:,1:nadd); % factor rotation for the pre-break sample
    
    temp_selJ = r2Jset(Ix);
    selJ      = [selJ; temp_selJ(1:nadd)]; %selected categories
    
    % Second sample factor explanation order:
    Ix2  = Ix;
    sR22 = sR2;
end

% post-break rotation matrix
if nadd>0
    Omega2_til = [C0, Cadd];
else
    Omega2_til = C0;
end

% Rotated factors and loadings
RF2_til      = F2_til*Omega2_til; % T1*r1
Rlambda2_til = (Omega2_til\lambda2_til')'; %n*r1


%% Factor identification (rotation) - Full sample
temp_R2  = zeros(J,1);
temp_bet = zeros(r2, J);

% loop over categories
for j=1:1:J
    jcat = Jset(j);
    
    xi  = sX(fXcatcode == jcat,:); % category j's data
    nxi = size(xi,1);
    
    % get the first pc for cat j
    [f0,eig,f00] = svd(xi'*xi);
    fxi          = f0(:,1)*sqrt(T);
    lxi          = xi*fxi/T;
    
    % OLS (without intercept)
    [bet, R2] = myols(fxi, F_til);
    
    % store
    temp_bet(:,j) = bet;
    temp_R2(j)    = R2;
    
end

% Select factor rotation based on the largest R2
[sR2, Ix]  = sort(temp_R2, 'descend');
sbet       = temp_bet(:,Ix);

selJfull   = Jset(Ix); % sort Jset as well

% Rotated factors and loadings
if r2<=J
Omega_til  = sbet(1:r2,1:r2); % factor rotation for the pre-break sample
RF_til      = F_til*Omega_til; % T*r2
Rlambda_til = (Omega_til\lambda_til')'; %n*r2
end
%% Table 0: Top 15 Correlations with rotated factors.
% full sample factor loading based on the Omega2 rotation
% RF_til00    = F_til*Omega2_til;
% tab_toplist00 = list_corr(RF_til00, sX', xname, xcatcode, r2, 15);
% tab_toplist = list_corr(RF_til, sX', xname, xcatcode, r2, 15);

% Using all six factors
% tab_toplist_pre  = list_corr(RF_til00(1:T1,:), X1', xname, xcatcode, r2, 15);
% tab_toplist_post = list_corr(RF_til00(T1+1:end,:), X2', xname, xcatcode, r2, 15);


tab_toplist_pre  = list_corr(RF1_til, X1', xname, xcatcode, r1, 15, xcatname);
tab_toplist_post = list_corr(RF2_til, X2', xname, xcatcode, min(r2,J), 15, xcatname);

% Save
cd(savepath);
xlswrite(tabname, tab_toplist_pre, 'Top15 correlation (pre)', 'A2');
xlswrite(tabname, tab_toplist_post, 'Top15 correlation (post)', 'A2');
cd(workpath);

%% Table 1
% split sample
tab0 = selJ; %selected cat number
tab1 = xcatname(selJ); %selected category names
tab2 = [ones(r1,1); 2*ones(min(r2,J)-r1,1)]; % pre/post break identifier

tab  = [num2cell(tab0), tab1, num2cell(tab2)];

% full sample
tabf0 = num2cell(selJfull);
tabf1 = xcatname(selJfull);
tabf  = [tabf0, tabf1];

disp('-----------------------------------');
disp('break at');
disp(xcal(T1))

disp('split sample');
disp(tab)

disp('full sample');
disp(tabf)

% Save
tab_head = {'', 'group', 'r1 or r2'};
tab00    = [tab_head; tab];
note     = {'ordered by R2'};
cd(savepath);
xlswrite(tabname, note, 'Label (break)', 'A1');
xlswrite(tabname, tab00, 'Label (break)', 'A2');
cd(workpath);

tabf_head = {'', 'group'};
tabf00    = [tabf_head; tabf];
note     = {'ordered by R2'};
cd(savepath);
xlswrite(tabname, note, 'Label (full sample)', 'A1');
xlswrite(tabname, tabf00, 'Label (full sample)', 'A2');
cd(workpath);


%% Table 2: R2 ordering
tab1_R2 = [xcatname(Jset(Ix1)) num2cell(sR21)];
tab2_R2 = [xcatname(r2Jset(Ix2)) num2cell(sR22)];

disp('----------------------------------------');
disp('Pre-break sample regression R2');
disp(tab1_R2);
disp('Post-break sample regression R2');
disp(tab2_R2);

% Save
tab_head  = {'Group', 'R2'};
tab1_R200 = [tab_head; tab1_R2];
note      = {'Pre-break regression: group specific factor versus aggregate factors'};
cd(savepath);
xlswrite(tabname, note, 'R2 (pre)', 'A1');
xlswrite(tabname, tab1_R200, 'R2 (pre)', 'A2');
cd(workpath);

tab_head  = {'Group', 'R2'};
tab2_R200 = [tab_head; tab2_R2];
note      = {'Post-break regression: group specific factor versus aggregate factors'};
cd(savepath);
xlswrite(tabname, note, 'R2 (post)', 'A1');
xlswrite(tabname, tab2_R200, 'R2 (post)', 'A2');
cd(workpath);


%% Category factor versus Aggregate factor
RFXXX   = [RF1_til; RF2_til(:,1:r1)]; % collect factors, aggregate
selname = strtrim(xcatname(selJ));

for i = 1:1:r1
    
    %set figure
    fig = figure(i);
    setmyfig(fig, [1.7, 1.2, 12, 5]);
    
    
    % current i's information
    temp_j     = selJ(i);
    temp_jname = selname{i};
    
    xi  = sX(fXcatcode == temp_j,:); % category j's data
    nxi = size(xi,1);
    
    % get the first pc for cat j
    [f0,eig,f00] = svd(xi'*xi);
    fxi          = f0(:,1)*sqrt(T);
    lxi          = xi*fxi/T;
    
    % correlation (X, X1, X2)
    temp_c1 = corr(fxi,RFXXX(:,i));
    temp_c2 = corr(fxi(1:T1,:),RFXXX(1:T1,i));
    temp_c3 = corr(fxi(T1+1:end,:),RFXXX(T1+1:end,i));
    
    if temp_c2<0
        fxi(1:T1,:) = -fxi(1:T1,:);
    elseif temp_c2<0
        fxi(T1+1:end,:) = -fxi(T1+1:end,:);
    end
    
    
    % draw plot
    plot(xcal, fxi, 'b', 'linewidth', 3) %individual factor
    hold on
    plot(xcal, RFXXX(:,i), 'r--', 'linewidth', 3) % aggregate factor
    shadenber
    line([xcal(T1) xcal(T1)], get(gca, 'ylim'), 'color', 'k', 'linewidth', 3); %break point line
%     l = legend('Group specific factor', 'Rotated aggregated factor');
    set(gca, 'linewidth', 2, 'fontsize', 25)
    hold off
    axis tight
    xlim([xcal(1)-0.5, xcal(end)+0.5])
%     set(l, 'fontsize', 15, 'location', 'northwest')
%     title(['Label : ', temp_jname, ', Corr(full,X1,X2) = ', ...
%            '(', num2str(temp_c1,2), ', ', num2str(temp_c2,2), ', ' num2str(temp_c3,2), ')'], 'fontsize', 20);
    
    % save
    cd(savepath)
    saveas(fig, ['fig_', specname, '_f', num2str(i), '_', temp_jname, '.png']);
    cd(workpath)
end

%% Category factor versus Aggregate factor (Draw only pre-break)
RFXXX   = [RF1_til; RF2_til(:,1:r1)]; % collect factors, aggregate
selname = strtrim(xcatname(selJ));

for i = 1:1:r1
    
    %set figure
    fig = figure(i);
    setmyfig(fig, [1.7, 1.2, 12, 5]);
    
    
    % current i's information
    temp_j     = selJ(i);
    temp_jname = selname{i};
    
    xi  = sX(fXcatcode == temp_j,:); % category j's data
    nxi = size(xi,1);
    
    % get the first pc for cat j
    [f0,eig,f00] = svd(xi'*xi);
    fxi          = f0(:,1)*sqrt(T);
    lxi          = xi*fxi/T;
    
    % correlation (X, X1, X2)
    temp_c1 = corr(fxi,RFXXX(:,i));
    temp_c2 = corr(fxi(1:T1,:),RFXXX(1:T1,i));
    temp_c3 = corr(fxi(T1+1:end,:),RFXXX(T1+1:end,i));
    
    if temp_c2<0
        fxi(1:T1,:) = -fxi(1:T1,:);
    elseif temp_c2<0
        fxi(T1+1:end,:) = -fxi(T1+1:end,:);
    end
    
    
    % draw plot
    fxi00 = nan*ones(size(fxi));
    fxi00(1:T1,:) = fxi(1:T1,:);
    RFXXX00 = nan*ones(size(RFXXX(:,i)));
    RFXXX00(1:T1,1) = RFXXX(1:T1,i);
    
    plot(xcal, fxi00, 'b', 'linewidth', 3) %individual factor
    hold on
    plot(xcal, RFXXX00, 'r--', 'linewidth', 3) % aggregate factor
    shadenber
    line([xcal(T1) xcal(T1)], get(gca, 'ylim'), 'color', 'k', 'linewidth', 3); %break point line
%     l = legend('Group specific factor', 'Rotated aggregated factor');
    set(gca, 'linewidth', 2, 'fontsize', 25)
    hold off
    axis tight
    xlim([xcal(1)-0.5, xcal(end)+0.5])
%     set(l, 'fontsize', 15, 'location', 'northwest')
%     title(['Label : ', temp_jname, ', Corr(full,X1,X2) = ', ...
%            '(', num2str(temp_c1,2), ', ', num2str(temp_c2,2), ', ' num2str(temp_c3,2), ')'], 'fontsize', 20);
    
    % save
    cd(savepath)
    saveas(fig, ['fig_', specname, '_f', num2str(i), '_', temp_jname, '_pre.png']);
    cd(workpath)
end


%% additional factor
for i=1:1:nadd
    
    % select group
    xi  = X2(fXcatcode == selJ(r1+i),:); % category j's data
    nxi = size(xi,1);    
    
    % set figure
    fig = figure(r1+i);
    setmyfig(fig, [1.7, 1.2, 12, 5]);

    % get the first pc for cat j
    [f0,eig,f00] = svd(xi'*xi);
    fxi          = f0(:,1)*sqrt(T2);
    lxi          = xi*fxi/T2;

    % draw plot
    plot(xcal(T1+1:end,:), fxi, 'b', 'linewidth', 3)
    hold on
    plot(xcal(T1+1:end,:), RF2_til(:,r1+i), 'r--', 'linewidth', 3)
    shadenber
    line([xcal(T1) xcal(T1)], get(gca, 'ylim'), 'color', 'k', 'linewidth', 3); %break point line
%     l = legend('Group specific factor', 'Rotated aggregated factor');
    set(gca, 'linewidth', 2, 'fontsize', 25)
    hold off
%     set(l, 'fontsize', 15, 'location', 'northeast')
%     title(['Label : ', selname{r1+i}, ' , Corr = ', num2str(corr(fxi, RF2_til(:,r1+i)),2)], 'fontsize', 20);
    axis tight
    xlim([xcal(T1)-0.5, xcal(end)+0.5])
    
    % save
    cd(savepath)
    saveas(fig, ['fig_', specname, '_f', num2str(r1+i), '_',selname{r1+i}, '.png']);
    cd(workpath)
end







