
% loop for detailed factor analysis

%% preliminary
clear all; clc; close all;


%% -------------------------------------------------------------------------
% SPECIFICATION
% - [Break date],[dataset],[kmax],[interval size],[rtc]
% -------------------------------------------------------------------------
% opt.rtc  = 4;  %reciprocal of tuning constant
opt.ver  = 0; % this option degenerated (as of 11/06/2013)
% 0: new lambda, new penalty (9/30/2013 version)
% 1: old program but new lam
% 2: old program but new penalty
% 3: old program
opt.kmax = 8; % maximum # factors
opt.d    = 21; %1: xdata_std, 2: xdata_uf_std (unfiltered)
% 1 : xdata_std, 
% 2 : xdata_uf_std   (unfiltered), 
% 11: NG's dataset  (same span as SW), unfiltered
% 12: NG's dataset  (longer span), unfiltered
% 21: SW version 03 (same span as SW), (bw = 100)
% 22: SW version 03 (same span as SW), unfiltered
% 23: SW version 03 (same span as SW), filtered (bw = 300)
% 31: SW version 03 (short), 
% 32: SW version 03 (short), unfiltered

opt.kill2007 = 0; %1 if you want to kill data after 2007 (only applies to 2001 recession)
% opt.bdate = '2001M03';
opt.bdate = '2007M12';

% opt.intsz  = 12;    % setting this to zero boils down to ``knowing break date'' situation.
% intsz_l    = intsz; % search interval size (lower)
% intsz_u    = intsz; % search interval size (upper)


% loop over rtc and intsz

loop.rtc.set   = [1, 4, 6];  loop.rtc.n = length(loop.rtc.set);
loop.intsz.set = [0, 6, 12]; loop.intsz.n = length(loop.intsz.set);
loop.result    = cell(1,opt.kmax+1+4);

for rtc_ind = 1:1:loop.rtc.n
    for intsz_ind = 1:1:loop.intsz.n
        
        % following specification changes over the loop
        opt.rtc    = loop.rtc.set(rtc_ind);
        opt.intsz  = loop.intsz.set(intsz_ind);
        intsz_l    = opt.intsz; % search interval size (lower)
        intsz_u    = opt.intsz; % search interval size (upper)
        specname = ['b', opt.bdate, '_d', num2str(opt.d), 'k', num2str(opt.kmax), 'i', num2str(opt.intsz), 't', num2str(opt.rtc),];

        % run script
        script_for_loop;
        
        % stack results
        loop.result = [loop.result; rstvec];
        
        % clear
        clearvars -except loop opt intsz_ind rtc_ind xcal
        
    end
end

%% Save to excel file
workpath = pwd;
savepath = [workpath, '\result_loop'];
chk_dir(savepath);

% table name
tabname  = ['ver', num2str(opt.ver), 'result_d', num2str(opt.d), 'bdate', opt.bdate, 'kmax', num2str(opt.kmax), 'kill', num2str(opt.kill2007), '.xls'];

% estimation info
note = [{'Estimation Sample', '', '', ''}; ...
    ['Estimation sample      : ', (xcaltrans(xcal(1))), ' ~ ', (xcaltrans(xcal(end)))]; ...
    ];
% result
head     = ['Specification', 'Factors', cell(1, opt.kmax-1), 'r1', 'r2', 'D', 'Bdate'];
finaltab = [head; loop.result];

cd(savepath);
xlswrite(tabname,note,'sample info','A2');
xlswrite(tabname,finaltab,'loop result','A2');
cd(workpath);




