% script to do analysis using loop
% design to do the similar analysis as in the main_full_part1 and main_full_part2
% only requires opt (option variable)
% Do not produce any graph/table

% 10/02/2013: add one more category (intR -> intS, intL)

%  10/14/2013: XC: change penalty in the first stage, change gamma penalty,
%  change ra and rb definition 


% %% -------------------------------------------------------------------------
% % SPECIFICATION
% % - [Break date],[dataset],[kmax],[interval size],[rtc]
% % -------------------------------------------------------------------------
% opt.rtc  = 4;  %reciprocal of tuning constant
% opt.kmax = 8; % maximum # factors
% opt.d    = 1; %1: xdata_std, 2: xdata_uf_std (unfiltered)
% 
% % T0    = 276; % conjectured break point
% % bdate = xcaltrans(xcal(T0)); %break date string
% 
% opt.bdate = '2007M12';
% 
% opt.intsz  = 12;    % setting this to zero boils down to ``knowing break date'' situation.
% intsz_l    = intsz; % search interval size (lower)
% intsz_u    = intsz; % search interval size (upper)

%% -------------------------------------------------------------------------
% Path control
% -------------------------------------------------------------------------
% current folder
workpath = pwd;

% add toolbox
path_original     = path;
path_toolbox1  = genpath([workpath, '\', 'toolbox_mc']);
path_toolbox2  = genpath([workpath, '\', 'toolbox_pc']);
addpath(path_toolbox1); 
addpath(path_toolbox2);

% datapath
datapath = [workpath, '\data'];


%% -------------------------------------------------------------------------
% Load DATA
% -------------------------------------------------------------------------
% data selection part 1
cd(datapath);
if opt.d <=10
    load('SW_xdata_ver02.mat'); % SW's data (old)
elseif (opt.d >10)&&(opt.d<=20)
    load('NG_xdata_ver02.mat'); % NG's data
elseif (opt.d == 21)||(opt.d == 22)
    load('SW_xdata_ver03_long.mat'); % SWver03 (long), bw=100
    elseif (opt.d == 23)
    load('SW_xdata_ver03_long_bw300.mat'); % SWver03 (long), bw=300
elseif (opt.d >30)&&(opt.d<=40) 
    load('SW_xdata_ver03_short.mat'); % SWver03 (short)
end
cd(workpath);

% data selection part 2
if (opt.d == 1)||(opt.d == 21)||(opt.d == 23)||(opt.d == 31) %filtered
    X = xdata_std'; %N by T form
elseif (opt.d == 2)||(opt.d == 22)||(opt.d == 32) %unfiltered
    X = xdata_uf_std'; %N by T form
elseif opt.d == 11
    X    = xdata_std(301:end,:)'; % N by T form, select 1985-2012 just as in SW
    xcal = xcal(301:end,:);
elseif opt.d == 12
    X = xdata_std(1:end,:)'; % N by T form, longer than SW, it starts from 1960
end

% if 2001M03 is pre-set break, we exclude post 2007 break data
if opt.kill2007 == 1
    if strcmp(opt.bdate,'2001M03')
        [X, xcal] = killpost2007(X,xcal);
        X=(X-repmat(mean(X,2),1,size(X,2)))./(repmat(std(X')',1,size(X,2)));
    end
end

[N,T] = size(X);

% catcode names
if (opt.d <=10)||(opt.d>20)
    xcatname = { ...
        'NIPA' ;...    %1'NIPA'
        'IP  ' ;...    %2'IP'
        'Emp ' ;...    %3'Emp/Unemp'
        'HSS ' ;...    %4'Hous.Starts'
        'Ord ' ;...    %5'Orders/Inv/Sales'
        'Pri ' ;...    %6'Prices'
        'Prod' ;...    %7'EmpEarning/Productivity'
        'IntL' ;...    %8'Int. Rates' Level
        'IntS' ;...    %9'Int. Rates' Spread
        'Mon ' ;...    %10'Money&Credit'
        'StPr' ;...    %11'StockPrices&Wealth'
        'HoPr' ;...    %12'Housing Prices'
        'ExR ' ;...    %13'Exch.Rates'
        '' ;...        %14'' not used
        'Others';...   %15'Other': 1) consumer expectation, 2) political uncertainty
        };
    % Int Rate: Level and spread
    xcatcode(xcatcode>9,1) = xcatcode(xcatcode>9,1) + 1;
    xcatcode(floor(xcatcode)==8,1) = xcatcode(floor(xcatcode)==8,1) + 1;
    xcatcode(xcatcode == 9.01,1) = 8.01;
    xcatcode(xcatcode == 9.02,1) = 8.02;
    % tab_xname = [num2cell((1:1:size(xname,1))') xname (xcatname(floor(xcatcode))) num2cell(xcatcode)];
       
elseif (opt.d >10)&&(opt.d<20)
    xcatname = { ...
        'Output ' ;...    %1'output'
        'Labor  ' ;...    %2'Labor'
        'Housing' ;...    %3'Housing'
        'Ord    ' ;...    %4'Orders/Inv/Sales'
        'Money  ' ;...    %5'Money&Credit'
        'Bond   ' ;...    %6'Bond'
        'Pri    ' ;...    %7'Prices'
        'StPr   ' ;...    %8'Stock market'
        'ExR    ' ;...    %9'Exchange rate'
        };
end

%% -------------------------------------------------------------------------
% Preliminary Setting
% -------------------------------------------------------------------------
% Penalization tuning constant (default is 1 for both)
%Fconst1 = 1/opt.rtc; %on Ftuning1
%Fconst2 = 1/opt.rtc; %on Ftuning2

% Back out T0 (conjectured break point)
temp_y    = str2double(opt.bdate(1:4)); %year
temp_m    = str2double(opt.bdate(6:7)); %month
temp_date = temp_y + (temp_m-1)/12;
temp_indx = (1:1:size(xcal,1))';
T0        = temp_indx(xcal == temp_date,1);

% Break region setting
% tao_0 = 272/T; % center of the break region to check
tao_0 = T0/T; % center of the break region to check
step  = 1/T;   % step size for window (1/T means monthly frequency step)

%region to search break dates
tao_l   = tao_0-step*intsz_l; %lower bound for the break region
tao_u   = tao_0+step*intsz_u; %upper bound for the break region
taov    = tao_l:step:tao_u; %vector of the break region
taovcal = xcal(int16(taov*T)'); %break region calendar

% maximum number of factors
k = opt.kmax;

% optimization option/construct group penality vector
options = optimset('MaxFunEvals', 200000, 'MaxIter', 500);
options.verbose = 1;
% options.optTol  = 1e-10;
% options.progTol = 1e-15;

groups  = zeros(2*N*k,1); %define the groups for group penalization
for i=1:k
    s=(i-1)*N+1; e=i*N;
    groups(s:e)=i;
    s=(k+(i-1))*N+1; e=(k+i)*N;
    groups(s:e)=k+i;
end

%% ------------------------------------------------------------------------
% PART 2: FACTOR LABELING
% ------------------------------------------------------------------------
%% ----------------------------------------------------------------
%---------------- Set tuning parameters----------------------------
%------------------------------------------------------------------
% ntao     = size(taov,2);
% TuningV  = zeros(2*k,ntao); %collecting tuning vector for searching range
% StartVal = zeros(2*N*k,ntao); %set-up starting value
% 
% for j = 1:ntao
%     
%     tao = taov(j);
%     T1  = floor(T*tao); T2=T-T1;    
%     
%     [Tuning, fpara0] = tuningpara(X,N,T1,T2,k,opt.rtc,groups,options, opt.ver);
%     
%     TuningV(:,j)  = Tuning; 
%     StartVal(:,j) = fpara0;
%     
% end
% 
% % Get tuning parameter and starting value (take average)
% Tuning = mean(TuningV,2);  % take average 
% fpara0 = mean(StartVal,2); % take average

% XC. New two-step method to set tuning parameter
cons=opt.rtc;
[TuningV,fpara0V,normlV,normgV] = tuningpara1(X,N,T,k,taov,cons);
Tuning = mean(TuningV,2);  % take average 
fpara0 = mean(fpara0V,2); % take average

[rv,paraestv,Dv] = plsest(X,N,T,k,taov,Tuning,fpara0,groups,options); %1st step estimation

r1_hat_V = rv(2,:);
r1_hat   = min(r1_hat_V);
r2_hat_V = rv(3,:);
r2_hat   = min(r2_hat_V);

TuningV = tuningpara2(X,N,T,k,taov,cons,r1_hat,r2_hat,normlV,normgV);
Tuning = mean(TuningV,2);  % take average 

[rv,paraestv,Dv] = plsest(X,N,T,k,taov,Tuning,fpara0,groups,options); %1st step estimation
r1_hat_V = rv(2,:);
r1_hat   = min(r1_hat_V);
r2_hat_V = rv(3,:);
r2_hat   = min(r2_hat_V);


D=norm(Dv); % end of model selection results, r1_hat, r2_hat, D

% %% -------------------------------------------------------------------------
% % Estimation (loop over the break-region)
% % - We fix "Tuning" at the average TuningV over the searching range
% % -------------------------------------------------------------------------
% rv       = zeros(3,ntao);
% paraestv = zeros(2*N*k,ntao);
% Dv       = zeros(1,ntao);
% %Resv     = zeros(1,ntao);
% for j = 1:ntao
%     
%     tao = taov(j);
%     T1  = floor(T*tao); T2=T-T1;
%     [F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1,T2,k); %principle component estimator of factors and loadings
%     funObj   = @(para)pel_ls(para,X,F1_til,F2_til);  %criterion function function
%     
%     %update shrinkage estimation using the new penalty
%     paraest = L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);
%     fest    = reshape(paraest,N,2*k); %updated shrinkage estimator
%     
%     A = diag(fest'*fest); A1 = A(1:k); A2 = A(k+1:2*k); %A2 is the difference.
%     r1_hat = find(A1,1,'last');
%     if norm(A2)==0
%         r2_hat = r1_hat;
%     else
%         r2_hat = find(A2,1,'last');
%         r2_hat = max(r1_hat,r2_hat);
%     end
%     
% 	% collect results
%     rv(:,j) = [r1_hat+r2_hat;r1_hat;r2_hat];
%     paraestv(:,j) = paraest;
%     
%     % some old stuff
%     fest    = reshape(paraest,N,2*k); %parameter matrix, 1:k is the 1st subsample, k+1:2k: difference
%     gamma   = fest(:,k+1:2*k); %difference in loading
%     D       = norm(gamma(:,1:r1_hat),'fro');
%     Dv(:,j) = D;
% 	
% end %end of loop for ntov
% 
% %% model selection. update 9/9/13
% r1_hat_V = rv(2,:);
% r1_hat   = min(r1_hat_V);
% r2_hat_V = rv(3,:);
% r2_hat   = min(r2_hat_V);
% 
% gamma_mat = paraestv(N*k+1:end,:);
% D         = norm(gamma_mat,'fro');


%% Break selection criteria - new
% - Choose the points closest to the conjecture point (among parsimous models)
rsum    = rv(1,:);
rset    = find(rsum==min(rsum));
rmid    = ceil(size(rv,2)+1)/2; %this assumes the conjecture point is in the middle of the searching interval
rbreak  = find(taov == tao_0); % exact conjectured break point

rdist   = abs(rset-rbreak);
[a1,a2] = min(rdist);

jr      = rset(a2);

% 
% r1_hat  = rv(2,jr);
% r2_hat  = rv(3,jr);


tao_hat = taov(jr); %break point chosen for model selection
bdate_hat = xcaltrans(xcal(floor(tao_hat*T))); %break date

% Other estimates at the selected break date
paraest = paraestv(:,jr);
fest    = reshape(paraest,N,2*k); %parameter matrix, 1:k is the 1st subsample, k+1:2k: difference
psi_est = fest(:,1:r2_hat)+fest(:,k+1:k+r2_hat); 

% Collect model selection result
% - decision vector and cell
decvec  = ['(', 'r1 = ', num2str(r1_hat), ', r2 = ', num2str(r2_hat), ', D = ', num2str(D,4), ', ', bdate_hat{:}, ')'];
deccell = {num2str(r1_hat), num2str(r2_hat), num2str(D,4), bdate_hat{:}};

%% Factor estimation at the estimated break date
T1_hat = floor(T*tao_hat);
T2_hat = T-T1_hat;
[F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1_hat,T2_hat,k);


%% Report results from our method
rslt_mat =[taovcal'; rv; Dv]'; %results for all break possible points
rslt_sel= [r1_hat,r2_hat,D];   %result that the method selects

disp(' ----------------- Estimation Matrix --------------------'); 
disp(['Potential break region : ', num2str(taovcal(1)), ' ~ ', num2str(taovcal(end))]);
disp(['Estimation sample      : ', num2str(xcal(1)), ' ~ ', num2str(xcal(end))]);
disp('---------------------------------------------------------');
disp('Model Selection Result (r1, r2, D)'); 
disp(['Break point at : ', num2str(xcal(int16(T*tao_hat)))]);
disp(rslt_sel);
disp('---------------------------------------------------------');

%% ------------------------------------------------------------------------
% PART 2: FACTOR LABELING
% ------------------------------------------------------------------------
% SOME INPUTS from the model selection procedure
T1 = floor(T*tao_hat);  %276: 2007Dec
T2 = T-T1;
r1 = r1_hat;
r2 = r2_hat;

%% step1: preliminary
% - sort data in xcatcode order
[sXcatcode, Ix] = sort(xcatcode, 'ascend');
sX = X(Ix,:);

fXcatcode = floor(sXcatcode); % floor
Jset      = unique(fXcatcode);     % unique categories
J         = length(Jset); % # of unique category number

if J<max(r1,r2)
    disp('max(r1,r2) is larger than # of categories');
%     disp('Stop the program ... ');
%     pause
end

Jfreq = histc(fXcatcode, Jset); % frequency of catcode

% disp('-------------------------------');
% disp('# of individual series in each category');
% disp([xcatname(Jset,1), num2cell(Jfreq)]);

%% step2: Factor estimation
X1 = sX(:,1:T1); %N*T1
X2 = sX(:,T1+1:T1+T2); %N*T2

% full sample
[f0,eig,f00] = svd(sX'*sX);
F_til        = f0(:,1:r2)*sqrt(T); %T*r2
lambda_til   = sX*F_til/T; %N*r1

% Pre-break
[f0,eig,f00] = svd(X1'*X1);
F1_til       = f0(:,1:r1)*sqrt(T1); %T1*r1
lambda1_til  = X1*F1_til/T1; %N*r1

% Post-break
[f0,eig,f00] = svd(X2'*X2);
F2_til       = f0(:,1:r2)*sqrt(T2); %T2*r2
lambda2_til  = X2*F2_til/T2; %N*r2


%% step3: Factor identification (rotation) - Pre-break sample
temp_R2  = zeros(J,1);
temp_bet = zeros(r1, J);

% loop over categories
for j=1:1:J
    jcat = Jset(j);
    
    xi  = X1(fXcatcode == jcat,:); % category j's data
    nxi = size(xi,1);
    
    % get the first pc for cat j
    [f0,eig,f00] = svd(xi'*xi);
    fxi          = f0(:,1)*sqrt(T1);
    lxi          = xi*fxi/T1;
    
    % OLS (without intercept)
    [bet, R2] = myols(fxi, F1_til);
    
    % store
    temp_bet(:,j) = bet;
    temp_R2(j)    = R2;
    
end

% Select factor rotation based on the largest R2
[sR2, Ix]  = sort(temp_R2, 'descend');
sbet       = temp_bet(:,Ix);
Omega1_til = sbet(1:r1,1:r1); % factor rotation for the pre-break sample

sJset    = Jset(Ix); % sort Jset as well
r2Jset   = sJset(r1+1:end); % kill cat that were used to construct F1
selJ     = sJset(1:r1); %selected categories

% Rotated factors and loadings
RF1_til      = F1_til*Omega1_til; % T1*r1
Rlambda1_til = (Omega1_til\lambda1_til')'; %n*r1

% First sample factor explanation order:
Ix1  = Ix;
sR21 = sR2;
% xcatname(Ix1)

%% Factor identification (rotation) - Post-break sample
% step 1: X2' = Rlambda1_til*F12_til + resid
% - Extending the rotated pre-break factor into the post-break period
F12_til = myols(X2, Rlambda1_til)';

% step 2: F12_til = F2*C0 + resid
C0 = myols(F12_til, F2_til);

% step 3 (only when r2>r1): We can use the variables groups j that were not
% used to construct RF1_til.

nadd     = min(r2-r1, J-r1); %# of additional factors
if nadd > 0
    
    temp_R2  = zeros(length(r2Jset),1);
    temp_bet = zeros(r2, length(r2Jset));
    
    for j = 1:1:length(r2Jset)
        jcat = r2Jset(j); %recall r2Jset is the leftover groups from the 1st analysis
        
        xi  = X2(fXcatcode == jcat,:); % category j's data
        nxi = size(xi,1);
        
        % get the first pc for cat j
        [f0,eig,f00] = svd(xi'*xi);
        fxi          = f0(:,1)*sqrt(T2);
        lxi          = xi*fxi/T2;
        
        % OLS (without intercept)
        [bet, R2] = myols(fxi, F2_til);
        
        % store
        temp_bet(:,j) = bet;
        temp_R2(j)    = R2;
        
    end
    
    % Select factor rotation based on the largest R2
    [sR2, Ix]  = sort(temp_R2, 'descend');
    sbet       = temp_bet(:,Ix);
    Cadd       = sbet(:,1:nadd); % factor rotation for the pre-break sample
    
    temp_selJ = r2Jset(Ix);
    selJ      = [selJ; temp_selJ(1:nadd)]; %selected categories
    
    % Second sample factor explanation order:
    Ix2  = Ix;
    sR22 = sR2;
end

% post-break rotation matrix
if nadd>0
    Omega2_til = [C0, Cadd];
else
    Omega2_til = C0;
end
% Rotated factors and loadings
RF2_til      = F2_til*Omega2_til; % T1*r1
Rlambda2_til = (Omega2_til\lambda2_til')'; %n*r1


%% Factor identification (rotation) - Full sample
% We use this for (r1=r2 and D=0) case
temp_R2  = zeros(J,1);
temp_bet = zeros(r2, J);

% loop over categories
for j=1:1:J
    jcat = Jset(j);
    
    xi  = sX(fXcatcode == jcat,:); % category j's data
    nxi = size(xi,1);
    
    % get the first pc for cat j
    [f0,eig,f00] = svd(xi'*xi);
    fxi          = f0(:,1)*sqrt(T);
    lxi          = xi*fxi/T;
    
    % OLS (without intercept)
    [bet, R2] = myols(fxi, F_til);
    
    % store
    temp_bet(:,j) = bet;
    temp_R2(j)    = R2;
    
end

% Select factor rotation based on the largest R2
[sR2, Ix]  = sort(temp_R2, 'descend');
sbet       = temp_bet(:,Ix);

selJfull   = Jset(Ix); % sort Jset as well

if r2<=J
    Omega_til  = sbet(1:r2,1:r2); % factor rotation for the pre-break sample
    
    % Rotated factors and loadings
    RF_til      = F_til*Omega_til; % T*r2
    Rlambda_til = (Omega_til\lambda_til')'; %n*r2
end

%% Table 1
% split sample
tab0 = selJ; %selected cat number
tab1 = xcatname(selJ); %selected category names
tab2 = [ones(r1,1); 2*ones(min(r2-r1,J-r1),1)]; % pre/post break identifier

tab  = [num2cell(tab0), tab1, num2cell(tab2)];

% full sample
tabf0 = num2cell(selJfull);
tabf1 = xcatname(selJfull);
tabf  = [tabf0, tabf1, num2cell(ones(size(tabf1,1),1))];
tabfr = tabf(1:min(r2,J),:);

% disp('-----------------------------------');
% disp('break at');
% disp(xcal(T1))
% 
% disp('split sample');
% disp(tab)
% 
% disp('full sample');
% disp(tabf)

% construct result vector (2 by kmax) cell matrix
if r2>r1
    rstvec00 = [{specname; 'r1 or r2'}, tab(:,2:3)', cell(2,opt.kmax-size(tab,1))];
elseif (r2==r1)&&(D~=0)
    rstvec00 = [{specname; 'r1 or r2'}, tab(:,2:3)', cell(2,opt.kmax-size(tab,1))];
elseif (r2==r1)&&(D==0)
    rstvec00 = [{specname; 'r1 or r2'}, tabfr(:,2:3)', cell(2,opt.kmax-size(tabfr,1))];
end

rstvec = [rstvec00, [deccell; cell(size(deccell))]];

