function R = maxcol(A,B)
%input: A: n by r; B: n by s, r>=s
%output: R: an r by r othorgonal rotation matrix
%        A*R=[A*R1:A*R2]
%        The collinearity between B and A*R1 is maximized by the choice of
%        R1
%        R2 is the null space of R1'


%        The measure of collinearity is trace(B*(A*R1)'), see Cliff(1966)
%        The notation in Cliff(1966) is different from that here, the
%        correspondence is as follows.
%        Cliff(1966) --> this program: F-->A, A-->B, U-->V, V-->U'

%        when r=s, this rotation also is the orthogonal procrustes
%        transformation


% example in Cliff(1966)
% B1 = [0.7;0.8;0.1;0];
% B2 = [0.1;0;0.7;0.8];
% B = [B1,B2];
% 
% A1=[0.76;0.5;0.52;0.5];
% A2=[0.32;0.5;-0.36;-0.5];
% A3=[0.5;-0.4;0.5;-0.4];
% A = [A1,A2,A3];


M=A'*B;
[U,S,V]=svd(M);
d = (S~=0);
S(d)=1;
Q = U*S*V';


R = [Q,null(Q')];


% below are some additional matrices of interest
AR = A*R;
B_hat = A*Q; %transformation of A best match B

%transformaiton of A and B with best match column by column
AT=A*U;
BT=B*V;

