function [mp,rp,rp2,mc,mr] = model_sel(N,T,T1,T2,r1,r2,k,eta1,eta2,rho1,rho2,a,beta,b,R,cons,R2_1,R2_2)

options=optimset('MaxFunEvals', 200000, 'MaxIter', 500);

ratio = 0.9.^(0:k); %ratio of variance across factors
share1=ratio(1:r1)/sum(ratio(1:r1)); %before break
share2=ratio(1:r2)/sum(ratio(1:r2));

groups=zeros(2*N*k,1); %define the groups for group penalization
for i=1:k   
s=(i-1)*N+1; e=i*N;   
groups(s:e)=i;
s=(k+(i-1))*N+1; e=(k+i)*N; 
groups(s:e)=k+i;  
end

%%%%%% End of Parameter Definition %%%%%%%%

Sm=zeros(R,3); Sr=zeros(R,4); Sc=zeros(R,4); 
parfor s=1:R  
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
[X,com] = paneldata(N,T1,T2,r1,r2,rho1,rho2,eta1,eta2,a,beta,b,R2_1,R2_2,share1,share2);

[F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1,T2,k); %principle component estimator of factors and loadings
funObj=@(para)pel_ls(para,X,F1_til,F2_til);  %criterion function function

[Tuning, fpara0] = tuningpara(X,N,T1,T2,k,cons,groups,options);

paraest=L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);
fest=reshape(paraest,N,2*k); %updated shrinkage estimator


A=diag(fest'*fest); A1=A(1:k); A2=A(k+1:2*k);
if norm(A1)==0
r1_hat=0;
else
r1_hat=find(A1,1,'last'); 
end

if norm(A2)==0
    r2_hat=r1_hat;
else
r2_hat=find(A2,1,'last');
r2_hat=max(r1_hat,r2_hat);
end

psi_est = fest(:,1:r2_hat)+fest(:,k+1:k+r2_hat); %shrinkage estimate of loading after break



%--------------------------------------------------------------
%% model selection results
if r1==r2 && b==0 && rho1==rho2 && eta1 == eta2
    model = 1;
elseif r2>r1
    model = 3;
else model = 2;
end

m1=0;m2=0; m3=0;
% lambda = fest(:,1:k);
gamma = fest(:,k+1:2*k);
D=norm(gamma(:,1:r1_hat),'fro');

if model == 1 %no structural change
if r1_hat==r1 && r2_hat == r2 && D==0
   m1=1;  %correct model  
elseif r1_hat==r2_hat && D==0
   m2=1;     
else     
   m3=1;    
end

end

if model == 2 %change in loadings, no new factor
if r1_hat ==r1 && r2_hat == r2 && D>0
   m1=1;  %correct model  
elseif r1_hat==r2_hat && D>0
   m2=1;     
else     
   m3=1;    
end
end

if model == 3 %new factors arise
if r1_hat == r1 && r2_hat == r2
   m1=1;  %correct model  
elseif r2_hat>r1_hat
   m2=1;     
else     
   m3=1;    
end

end

Sm(s,:)=[m1,m2,m3];

%----------------------------------------------------------------------
%fit statistics reported

% com2 = com(:,T1+1:end);
% 
% %mse of the shrinkage estimate of loadings and subsample of factors
% com2_se2 = psi_est*F2_til(:,1:r2_hat)'; %only use second sample
% 
% %common component after structural change selection
% if r1_hat == r2_hat && D==0; %no structural change    
%     [F_hat,lambda_hat] = pc_full(X,T,r1_hat);
%     com_est = lambda_hat*F_hat'; %NxT
%     com2_est = com_est(:,T1+1:end);
%     
%     % use shrinkage estimate of loadings
%     com2_se1 = com2_est;
%     
% else
%     [F2_hat,lambda2_hat] = pc_full(X2,T2,r2_hat);    
%     com2_est = lambda2_hat*F2_hat'; %NxT2
%     
%     com2_se1 = com2_se2;
%     
% end
% dif_est = com2_est-com2; %NxT2
% ndif_est=diag(dif_est*dif_est');
% Sc_est = mean(ndif_est); %measure for second subsample based on model selection
% 
% 
% 
% dif_se1 = com2_se1-com2; %NxT2
% ndif_se1=diag(dif_se1*dif_se1');
% Sc_se1 = mean(ndif_se1); %measure for second subsample based on model selection

% dif_se2 = com2_se2-com2; %NxT2
% ndif_se2=diag(dif_se2*dif_se2');
% Sc_se2 = mean(ndif_se2); %measure for second subsample based on model selection
% 
% %full sample with IC
% jj=2; %ICP2 in Bai-Ng (2002)
% DEMEAN=2; %standarize data
% [r_fic,lamda_hat,F_hat]=nbplog(X',k,jj,DEMEAN);
% 
% [F_hat,lambda_hat] = pc_full(X,T,r_fic);
% com_f = lambda_hat*F_hat'; %NxT
% com2_f = com_f(:,T1+1:end);
% dif_f = com2_f-com2; %NxT2
% ndif_f=diag(dif_f*dif_f');
% Sc_fic = mean(ndif_f); 


% %2nd subsample with IC
% [r2_sic,lamda_hat,F_hat]=nbplog(X2',k,jj,DEMEAN);
% [F_hat,lambda_hat] = pc_full(X2,T2,r2_sic);
% com2_s = lambda_hat*F_hat'; %NxT
% dif_s = com2_s-com2; %NxT2
% ndif_s=diag(dif_s*dif_s');
% Sc_sic = mean(ndif_s); 

X2 = X(:,T1+1:end);
X2_hat = X2; %known break date
T1_hat = T1; %known break date
T11 = T1; % search at one point

[Sc_est, Sc_se1, Sc_fic, Sc_sic, r_fic, r2_sic]= ...
    fit_mse(X,X2,X2_hat,F2_til,com,T,T11,T1_hat,k,r1_hat,r2_hat,psi_est,D);


Sc(s,:) = [Sc_est, Sc_se1, Sc_fic, Sc_sic];

Sr(s,:) = [r1_hat, r2_hat, r_fic, r2_sic]; %number of factors before and after break

% Sr_full(s,:)=[r_full,r1_sub,r2_sub];

end

%model statistics reported
mp = mean(Sm); %1x3 row vector
mc = mean(Sc);
mr = mean(Sr);
mc = mc/mc(1); %normlization

rp_s = mean(Sr(:,1)==(r1-1));
rp_e = mean(Sr(:,1)==r1);
rp_l = mean(Sr(:,1)==(r1+1));
rp = [rp_e,rp_s,rp_l]; %1x3 row vector

rp2_s = mean(Sr(:,2)==(r2-1));
rp2_e = mean(Sr(:,2)==r2);
rp2_l = mean(Sr(:,2)==(r2+1));
rp2 = [rp2_e,rp2_s,rp2_l]; %1x3 row vector

% %use full sample and subsample estimation
% 
% mr_full = mean(Sr_full);
% 
% rp_full_s = mean(Sr_full(:,1)==(r1+r2-1));
% rp_full_e = mean(Sr_full(:,1)==r1+r2);
% rp_full_l = mean(Sr_full(:,1)==(r1+r2+1));
% rp_full = [rp_full_e,rp_full_s,rp_full_l]; %1x3 row vector
% 
% rp_sub1_s = mean(Sr_full(:,2)==(r1-1));
% rp_sub1_e = mean(Sr_full(:,2)==r1);
% rp_sub1_l = mean(Sr_full(:,2)==(r1+1));
% rp_sub1 = [rp_sub1_e,rp_sub1_s,rp_sub1_l]; %1x3 row vector
% 
% rp_sub2_s = mean(Sr_full(:,3)==(r2-1));
% rp_sub2_e = mean(Sr_full(:,3)==r2);
% rp_sub2_l = mean(Sr_full(:,3)==(r2+1));
% rp_sub2 = [rp_sub2_e,rp_sub2_s,rp_sub2_l]; %1x3 row vector
% 
% 
% full = [mr_full,rp_full,rp_sub1,rp_sub2];




                                                                                                                                                                                                         