function [mp,rp,rp2,mc,mr,full] = model_sel(N,T,T1,T2,r1,r2,k,eta1,eta2,rho1,rho2,a,beta,b,R,cons,R2_1,R2_2)

options=optimset('MaxFunEvals', 200000, 'MaxIter', 500);

ratio = 0.9.^(0:k); %ratio of variance across factors
share1=ratio(1:r1)/sum(ratio(1:r1)); %before break
rat2 = [ratio(1:r1),ratio(r1)*ones(r2-r1)];
share2=rat2/sum(rat2);

groups=zeros(2*N*k,1); %define the groups for group penalization
for i=1:k   
s=(i-1)*N+1; e=i*N;   
groups(s:e)=i;
s=(k+(i-1))*N+1; e=(k+i)*N; 
groups(s:e)=k+i;  
end

%%%%%% End of Parameter Definition %%%%%%%%

Sm=zeros(R,3); Sr=zeros(R,4); Sc=zeros(R,3); Sr_full=zeros(R,3);
parfor s=1:R  
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
[X,com] = paneldata(N,T1,T2,r1,r2,rho1,rho2,eta1,eta2,a,beta,b,R2_1,R2_2,share1,share2);
[F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1,T2,k); %principle component estimator of factors and loadings
X1=X(:,1:T1); X2=X(:,T1+1:T);
R1=X1-lambda1_til*F1_til'; R2=X2-lambda2_til*F2_til'; %preliminary estimator of residuals

%initial tuning parameter
e1=norm(R1,'fro')/(N*T1)^.5+norm(R2,'fro')/(N*T2)^.5;
e2=norm(R2,'fro')/(N*T2)^.5;

dp=2;
Ftuning10=1/cons*N^(-.5)*max((T1)^(-(dp+1)/2),N^(-(dp+1)/2)); %a constant in the tuning parameter for penalization
Ftuning20=1/cons*N^(-.5)*max((T2)^(-(dp+1)/2),N^(-(dp+1)/2)); %a constant in the tuning parameter for penalization

norml = sqrt(diag(lambda1_til'*lambda1_til))'; %column norms of Lambda1
gamma_til=lambda2_til-lambda1_til; 
normg = sqrt(diag(gamma_til'*gamma_til))'; %column norms of Gamma
Ftuning1=e1*Ftuning10*N^dp*(norml.^(-2*dp));
Ftuning2=e2*Ftuning20*N^dp*(normg.^(-2*dp));

%shrinkage estimation based on initial tuning parameter
funObj=@(para)pel_ls(para,X,F1_til,F2_til);  %criterion function function
fpara=[lambda1_til gamma_til];
fpara0=reshape(fpara,2*N*k,1); %starting values
Tuning=[Ftuning1';Ftuning2'];
paraest0=L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);
fest0=reshape(paraest0,N,2*k); %preliminary estimator
A=diag(fest0'*fest0); B=fest0(:,1:k)+fest0(:,k+1:2*k); C=diag(B'*B);
er1=nnz(A(1:k)); er2=nnz(C); estr=max(er1,er2);
 
%update penalty based on preliminary estimator
F1_se = F1_til(:,1:estr);
F2_se = F2_til(:,1:estr);
lambda1_se = lambda1_til(:,1:estr);
lambda2_se = lambda2_til(:,1:estr);
R1_se=X1-lambda1_se*F1_se'; R2_se=X2-lambda2_se*F2_se';
e1=norm(R1_se,'fro')/(N*T1)^.5+norm(R2_se,'fro')/(N*T2)^.5;
e2=norm(R2_se,'fro')/(N*T2)^.5; 



[F2_tr,lambda2_tr] = rotation_lamb2(F2_se,lambda1_se,lambda2_se,er1); %rotate lambda2_se to update norm of gamma
gamma_se=lambda2_tr-lambda1_se; 
normg_se = sqrt(diag(gamma_se'*gamma_se))'; %column norms of Gamma_se
normg_n = normg;
normg_n(1:estr)=normg_se; %update norm

Ftuning1=e1*Ftuning10*(N^2)*(norml.^(-4));
Ftuning2=e2*Ftuning20*(N^2)*(normg_n.^(-4));
Tuning=[Ftuning1';Ftuning2'];

%update shrinkage estimation using the new penalty
paraest=L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);
fest=reshape(paraest,N,2*k); %updated shrinkage estimator


A=diag(fest'*fest); A1=A(1:k); A2=A(k+1:2*k);
if norm(A1)==0
r1_hat=0;
else
r1_hat=find(A1,1,'last'); 
end

if norm(A2)==0
    r2_hat=r1_hat;
else
r2_hat=find(A2,1,'last');
r2_hat=max(r1_hat,r2_hat);
end


%full sample estimation
r_full = rank_est_full(X,k,cons);
r1_sub = rank_est_full(X1,k,cons);
r2_sub = rank_est_full(X2,k,cons);

%--------------------------------------------------------------
%% model selection results
if r1==r2 && b==0 && rho1==rho2 && eta1 == eta2
    model = 1;
elseif r2>r1
    model = 3;
else model = 2;
end

m1=0;m2=0; m3=0;
% lambda = fest(:,1:k);
gamma = fest(:,k+1:2*k);
D=norm(gamma(:,1:r1_hat),'fro');

if model == 1 %no structural change
if r1_hat==r1 && r2_hat == r2 && D==0
   m1=1;  %correct model  
elseif r1_hat==r2_hat && D==0
   m2=1;     
else     
   m3=1;    
end

end

if model == 2 %change in loadings, no new factor
if r1_hat ==r1 && r2_hat == r2 && D>0
   m1=1;  %correct model  
elseif r1_hat==r2_hat && D>0
   m2=1;     
else     
   m3=1;    
end
end

if model == 3 %new factors arise
if r1_hat == r1 && r2_hat == r2
   m1=1;  %correct model  
elseif r2_hat>r1_hat
   m2=1;     
else     
   m3=1;    
end

end

Sm(s,:)=[m1,m2,m3];

%----------------------------------------------------------------------
%fit statistics reported
com2 = com(:,T1+1:end);

%common component after structural change selection
if r1_hat == r2_hat && D==0; %no structural change    
    [F_hat,lambda_hat] = pc_full(X,T,r1_hat);
    com_est = lambda_hat*F_hat'; %NxT
    com2_est = com_est(:,T1+1:end);
else
    [F2_hat,lambda2_hat] = pc_full(X2,T2,r2_hat);    
    com2_est = lambda2_hat*F2_hat'; %NxT2
end
dif_est = com2_est-com2; %NxT2
ndif_est=diag(dif_est*dif_est');
% Sc_est = mean(ndif_est./ncom); %measure for second subsample based on model selection
Sc_est = mean(ndif_est); %measure for second subsample based on model selection


%full sample with IC
jj=2; %ICP2 in Bai-Ng (2002)
DEMEAN=2; %standarize data
[r_fic,lamda_hat,F_hat]=nbplog(X',k,jj,DEMEAN);

[F_hat,lambda_hat] = pc_full(X,T,r_fic);
com_f = lambda_hat*F_hat'; %NxT
com2_f = com_f(:,T1+1:end);
dif_f = com2_f-com2; %NxT2
ndif_f=diag(dif_f*dif_f');
% Sc_fic = mean(ndif_f./ncom); %measure for second subsample based on model selection
Sc_fic = mean(ndif_f); 


%2nd subsample with IC
[r2_sic,lamda_hat,F_hat]=nbplog(X2',k,jj,DEMEAN);
[F_hat,lambda_hat] = pc_full(X2,T2,r2_sic);
com2_s = lambda_hat*F_hat'; %NxT
dif_s = com2_s-com2; %NxT2
ndif_s=diag(dif_s*dif_s');
% Sc_sic = mean(ndif_s./ncom); %measure for second subsample based on model selection
Sc_sic = mean(ndif_s); 

Sc(s,:) = [Sc_est, Sc_fic, Sc_sic];

Sr(s,:) = [r1_hat, r2_hat, r_fic, r2_sic]; %number of factors before and after break

Sr_full(s,:)=[r_full,r1_sub,r2_sub];

end

%model statistics reported
mp = mean(Sm); %1x3 row vector
mc = mean(Sc);
mr = mean(Sr);
mc = mc/mc(1); %normlization

rp_s = mean(Sr(:,1)==(r1-1));
rp_e = mean(Sr(:,1)==r1);
rp_l = mean(Sr(:,1)==(r1+1));
rp = [rp_e,rp_s,rp_l]; %1x3 row vector

rp2_s = mean(Sr(:,2)==(r2-1));
rp2_e = mean(Sr(:,2)==r2);
rp2_l = mean(Sr(:,2)==(r2+1));
rp2 = [rp2_e,rp2_s,rp2_l]; %1x3 row vector

%use full sample and subsample estimation

mr_full = mean(Sr_full);

rp_full_s = mean(Sr_full(:,1)==(r1+r2-1));
rp_full_e = mean(Sr_full(:,1)==r1+r2);
rp_full_l = mean(Sr_full(:,1)==(r1+r2+1));
rp_full = [rp_full_e,rp_full_s,rp_full_l]; %1x3 row vector

rp_sub1_s = mean(Sr_full(:,2)==(r1-1));
rp_sub1_e = mean(Sr_full(:,2)==r1);
rp_sub1_l = mean(Sr_full(:,2)==(r1+1));
rp_sub1 = [rp_sub1_e,rp_sub1_s,rp_sub1_l]; %1x3 row vector

rp_sub2_s = mean(Sr_full(:,3)==(r2-1));
rp_sub2_e = mean(Sr_full(:,3)==r2);
rp_sub2_l = mean(Sr_full(:,3)==(r2+1));
rp_sub2 = [rp_sub2_e,rp_sub2_s,rp_sub2_l]; %1x3 row vector


full = [mr_full,rp_full,rp_sub1,rp_sub2];




                                                                                                                                                                                                         