function [mp,rp,rp2,mc,mr,mj,mr_tot] = model_sel_gen(N,T,tao0,taov,r1,r2,k,eta1,eta2,rho1,rho2,a,beta,b,R,cons,R2_1,R2_2,brkpt)
% edited 08/20/13
% average tuning and starting value, new program tuningpara replace the
% codes below.


options=optimset('MaxFunEvals', 200000, 'MaxIter', 500);

ratio = 0.9.^(0:k); %ratio of variance across factors
share1=ratio(1:r1)/sum(ratio(1:r1)); %before break
share2=ratio(1:r2)/sum(ratio(1:r2));

groups=zeros(2*N*k,1); %define the groups for group penalization
for i=1:k   
s=(i-1)*N+1; e=i*N;   
groups(s:e)=i;
s=(k+(i-1))*N+1; e=(k+i)*N; 
groups(s:e)=k+i;  
end

%%%%%% End of Parameter Definition %%%%%%%%
ntao     = size(taov,2);
Sm=zeros(R,3); Sr=zeros(R,4); 
Sj=zeros(R,1);
Sr_tot = zeros(R,3*ntao); 
Sc=zeros(R,4);

parfor s=1:R  
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
T1=floor(T*tao0); T2=T-T1;
[X,com] = paneldata(N,T1,T2,r1,r2,rho1,rho2,eta1,eta2,a,beta,b,R2_1,R2_2,share1,share2);

%% ----------------------------------------------------------------
%---------------- set tuning parameters and Starting Value--------
%------------------------------------------------------------------

TuningV=zeros(2*k,ntao);
StartVal=zeros(2*N*k,ntao);

for j = 1:ntao
    
    tao = taov(j);
    T1  = floor(T*tao); T2=T-T1;       
    
    [Tuning, fpara0] = tuningpara(X,N,T1,T2,k,cons,groups,options);
   
    TuningV(:,j)=Tuning; 
    StartVal(:,j)=fpara0;
    
    
end

Tuning = mean(TuningV,2);
fpara0 = mean(StartVal,2);

 

%% ------------------------------------------------------------------------
% Estimation (loop over the break-region)
% -------------------------------------------------------------------------

rv       = zeros(3,ntao);
paraestv = zeros(2*N*k,ntao);


for j = 1:ntao
    
    tao = taov(j);
    T1  = floor(T*tao); T2=T-T1;
    [F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1,T2,k); %principle component estimator of factors and loadings
    funObj   = @(para)pel_ls(para,X,F1_til,F2_til);  %criterion function function
    
    %update shrinkage estimation using the new penalty
    paraest = L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);
    fest    = reshape(paraest,N,2*k); %updated shrinkage estimator
    
    A = diag(fest'*fest); A1 = A(1:k); A2 = A(k+1:2*k); %A2 is the difference.
    r1_hat = find(A1,1,'last');
    if norm(A2)==0
        r2_hat = r1_hat;
    else
        r2_hat = find(A2,1,'last');
        r2_hat = max(r1_hat,r2_hat);
    end
    
    rv(:,j) = [r1_hat+r2_hat;r1_hat;r2_hat]; % rv is 3xntao
    paraestv(:,j) = paraest;
    
    
    
end %end of loop for ntov

Sr_tot(s,:)=reshape(rv,1,3*ntao); %size of rv is ntao*3


%% model selection. update 9/9/13
r1_hat_V = rv(2,:);
r1_hat = min(r1_hat_V);
r2_hat_V = rv(3,:);
r2_hat = min(r2_hat_V);

gamma_mat = paraestv(N*k+1:end,:);
D = norm(gamma_mat,'fro');


%% Break selection criteria
% **********This is the model selection method based on conjecture date*****
rsum   = rv(1,:);
rset = find(rsum==min(rsum));

if brkpt == 1
% - Choose the points closest to the conjecture point
rmid=ceil(size(rv,2)+1)/2; %this assumes the conjecture point is in the middle of the searching interval
rdist=abs(rset-rmid);
[a1,a2]=min(rdist);
jr=rset(a2);

else
jr = max(rset); %the right end point
end

tao_hat = taov(jr); %break point chosen for model selection
% r1_hat  = rv(2,jr);
% r2_hat  = rv(3,jr);
paraest = paraestv(:,jr);
fest    = reshape(paraest,N,2*k); %parameter matrix, 1:k is the 1st subsample, k+1:2k: difference
psi_est = fest(:,1:r2_hat)+fest(:,k+1:k+r2_hat); 

% gamma   = fest(:,k+1:2*k); %difference in loading
% D       = norm(gamma(:,1:r1_hat),'fro');

%--------------------------------------------------------------
%% model selection results
if r1==r2 && b==0 && rho1==rho2 && eta1 == eta2
    model = 1;
elseif r2>r1
    model = 3;
else model = 2;
end

m1=0;m2=0; m3=0;
if model == 1 %no structural change

if r1_hat==r1 && r2_hat == r2 && D==0
   m1=1;  %correct model  
elseif r1_hat==r2_hat && D==0
   m2=1;     
else     
   m3=1;    
end

end

if model == 2 %change in loadings, no new factor

if r1_hat ==r1 && r2_hat == r2 && D>0
   m1=1;  %correct model  
elseif r1_hat==r2_hat && D>0
   m2=1;     
else     
   m3=1;    
end
end

if model == 3 %new factors arise

if r1_hat == r1 && r2_hat == r2
   m1=1;  %correct model  
elseif r2_hat>r1_hat
   m2=1;     
else     
   m3=1;    
end

end

Sm(s,:)=[m1,m2,m3];



%----------------------------------------------------------------------
%fit statistics reported
T10 = floor(T*tao0); %know the location of the break
X2 = X(:,T10+1:end);

T11 = floor(T*taov(end)); % last point considered for break
T1_hat = floor(T*tao_hat);
T2_hat = T-T1_hat;
X2_hat = X(:,T1_hat+1:end);

[F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1_hat,T2_hat,k);

[Sc_est, Sc_se1, Sc_fic, Sc_sic, r_fic, r2_sic]= ...
    fit_mse(X,X2,X2_hat,F2_til,com,T,T11,T1_hat,k,r1_hat,r2_hat,psi_est,D);

Sc(s,:) = [Sc_est, Sc_se1, Sc_fic, Sc_sic];
Sr(s,:) = [r1_hat, r2_hat, r_fic, r2_sic]; %number of factors before and after break
Sj(s,:)= tao_hat;


% 
% 
% 
% com2 = com(:,T11+1:end);
% 
% %common component after structural change selection
% if r1_hat == r2_hat && D==0; %no structural change    
%     [F_hat,lambda_hat] = pc_full(X,T,r1_hat);
%     com_est = lambda_hat*F_hat'; %NxT
%     com2_est = com_est(:,end-T22+1:end);
%     
%     
% else
%     T1_hat = floor(T*tao_hat);
%     T2_hat = T-T1_hat;
%     X2 = X(:,T1_hat+1:end);
%     [F2_hat,lambda2_hat] = pc_full(X2,T2_hat,r2_hat);    
%     com2_est = lambda2_hat*F2_hat'; %NxT2    
%     com2_est = com2_est(:,end-T22+1:end);
% 
% end
% 
% dif_est = com2_est-com2; %NxT2
% ndif_est=diag(dif_est*dif_est');
% Sc_est = mean(ndif_est); %measure for second subsample based on model selection
% 
% %mse of the shrinkage estimate of loadings and subsample of factors
% [F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1_hat,T2_hat,k);
% com2_se = psi_est*F2_til(end-T22+1:end,1:r2_hat)'; %only use second sample
% 
% dif_se = com2_se-com2; %NxT2
% ndif_se=diag(dif_se*dif_se');
% Sc_se = mean(ndif_se); %measure for second subsample based on model selection
% 
% 
% 
% %full sample with IC
% jj=2; %ICP2 in Bai-Ng (2002)
% DEMEAN=2; %standarize data
% [r_fic,lamda_hat,F_hat]=nbplog(X',k,jj,DEMEAN);
% 
% [F_hat,lambda_hat] = pc_full(X,T,r_fic);
% com_f = lambda_hat*F_hat'; %NxT
% com2_f = com_f(:,end-T22+1:end);
% dif_f = com2_f-com2; %NxT2
% ndif_f=diag(dif_f*dif_f');
% Sc_fic = mean(ndif_f); 
% 
% 
% %2nd subsample with IC
% T10 = floor(T*tao0); %know the location of the break
% T20=T-T10;
% X2 = X(:,T10+1:end);
% [r2_sic,lamda_hat,F_hat]=nbplog(X2',k,jj,DEMEAN);
% [F_hat,lambda_hat] = pc_full(X2,T20,r2_sic);
% com2_s = lambda_hat*F_hat'; %NxT
% com2_s = com2_s(:,end-T22+1:end);
% dif_s = com2_s-com2; %NxT2
% ndif_s=diag(dif_s*dif_s');
% Sc_sic = mean(ndif_s); 





end

mr_tot = mean(Sr_tot);
% mr_tot = reshape(mr_tot,3,ntao); %average number of factors for different break dates


%model statistics reproted
mp = mean(Sm); %1x3 row vector
mr = mean(Sr);
mj = mean(Sj); 

mc = mean(Sc);
mc = mc/mc(1); %normlization



rp_s = mean(Sr(:,1)==(r1-1));
rp_e = mean(Sr(:,1)==r1);
rp_l = mean(Sr(:,1)==(r1+1));
rp = [rp_e,rp_s,rp_l]; %1x3 row vector

rp2_s = mean(Sr(:,2)==(r2-1));
rp2_e = mean(Sr(:,2)==r2);
rp2_l = mean(Sr(:,2)==(r2+1));
rp2 = [rp2_e,rp2_s,rp2_l]; %1x3 row vector






                                                                                                                                                                                                         