function [X,C] = paneldata(N,T1,T2,r1,r2,rho1,rho2,eta1,eta2,a,beta,b,R2_1,R2_2,share1,share2)
%% This function generates the panel data with a break
% N: number of series
% T1: number of periods before the break
% T2: ..... after the break
% r1: number of factors before the break
% r2: ..... after the break
% rho1: AR(1) coeff of factor dynamicbefore break
% rho2: ..... after the break
% eta1: standard deviation of factor innovation before break
% eta2: ..... after the break
% a: AR(1) coeff of the idiosycratic errors
% R2_i: Nx1, R^2 for each series

% X: NxT panel data
% C: NxT common component


%% factor, Txr2 
u =  eta1*randn(T1,r2); %factor innovations with std eta1
f0 = randn(1,r2)/sqrt(1-rho1^2)*eta1; %initial values for factors
F = filter(1,[1 -rho1],[f0;u]); %AR(1) with coefficient rho1
F1 = F(2:end,1:r1);

u =  eta2*randn(T2,r2); %factor innovations with std eta2
f0 = F(end,:);
F2 = filter(1,[1 -rho2],[f0;u]); %AR(1) with coefficient rho2
F2 = F2(2:end,:);

%% idiosyncratic errors, TxN
% Calculate the matrix square root of cov matrix of the idiosyncratic errors
% Omega = (beta^{|i-j|})_{i,j}, same as in BPSW(2012)
circul = reshape(mod(0:N^2-1,N+1),N,N);
uind = (triu(ones(N))==1);
circul_tp = circul';
circul(uind) = circul_tp(uind); %matrix (|i-j|)_{i,j}
Omega = beta.^circul; %Define Omega
[OV,OD]=eig(Omega);
sqrtOmega = OV*sqrt(OD); %Matrix square root: sqrtOmega*sqrtOmega'=Omega

v = randn(T1+T2,N)*sqrtOmega'; %idosyncratic innovations
e0 = randn(1,N)*sqrtOmega'/sqrt(1-a^2); %initial value for idiosyncratic errors
e = filter(1,[1 -a],[e0;v]); %AR(1)with coefficient a
e = e(2:end,:);
e=e'; %NXT
 

lambda_star1 = @(x)((1-rho1^2)/(1-a^2)/eta1^2*x./(1-x)); %lambda^*(R^2),total variance
lambda1_share = sqrt(lambda_star1(R2_1)*share1); %Nxr1 vector, sqrt for std dev
lambda_0_bar = randn(N,r1);
size(lambda_0_bar)
size(lambda1_share)
lambda1 = lambda_0_bar.*lambda1_share; %initial loadings

if r2>r1 
        lambda_star2 = @(x)((1-rho2^2)/(1-a^2)/eta2^2*x./(1-x)); %lambda^*(R^2),total variance
        lambda2_share = sqrt(lambda_star2(R2_2)*share2); %Nxr2 vector, sqrt for std dev
        %lambda_0_bar = [0.5*lambda_0_bar+sqrt(1-0.5^2)*randn(N,r1),randn(N,r2-r1)];
        lambda_0_bar=randn(N,r2);
        lambda2 = lambda_0_bar.*lambda2_share; %2nd loadings 
        %lambda2= [randn(N,r1).*lambda1_share,randn(N,1).*lambda1_share(1)];
         
        
elseif  r1==r2 && rho1==rho2 && eta1==eta2; %dynmiac structure of the factors change
    
        lambda_0_bar = randn(N,r1); %change of loadings
        lambda2 = lambda1*(1-b)+lambda_0_bar.*lambda1_share*b;
    
else
        lambda2=lambda1;   
end

%% generate observed panel data
C1 = lambda1*F1';
C2 = lambda2*F2';
C=[C1,C2];
X=C+e;

T=T1+T2;
stx = repmat(std(X,[],2),1,T);
X=X./stx;
C=C./stx;

