function [X,C] = paneldata_full(N,T,r1,rho1,eta1,a,beta,R2_1,share1)
%% This function generates the panel data with a break
% N: number of series
% T1: number of periods before the break
% T2: ..... after the break
% r1: number of factors before the break
% r2: ..... after the break
% rho1: AR(1) coeff of factor dynamicbefore break
% rho2: ..... after the break
% eta1: standard deviation of factor innovation before break
% eta2: ..... after the break
% a: AR(1) coeff of the idiosycratic errors
% R2_i: Nx1, R^2 for each series

% X: NxT panel data
% C: NxT common component


%% factor, Txr2 
u =  eta1*randn(T,r1); %factor innovations with std eta1
f0 = randn(1,r1)/sqrt(1-rho1^2)*eta1; %initial values for factors
F = filter(1,[1 -rho1],[f0;u]); %AR(1) with coefficient rho1
F1 = F(2:end,1:r1);



%% idiosyncratic errors, TxN
% Calculate the matrix square root of cov matrix of the idiosyncratic errors
% Omega = (beta^{|i-j|})_{i,j}, same as in BPSW(2012)
circul = reshape(mod(0:N^2-1,N+1),N,N);
uind = (triu(ones(N))==1);
circul_tp = circul';
circul(uind) = circul_tp(uind); %matrix (|i-j|)_{i,j}
Omega = beta.^circul; %Define Omega
[OV,OD]=eig(Omega);
sqrtOmega = OV*sqrt(OD); %Matrix square root: sqrtOmega*sqrtOmega'=Omega

v = randn(T,N)*sqrtOmega'; %idosyncratic innovations
e0 = randn(1,N)*sqrtOmega'/sqrt(1-a^2); %initial value for idiosyncratic errors
e = filter(1,[1 -a],[e0;v]); %AR(1)with coefficient a
e = e(2:end,:);
e=e'; %NXT
 

lambda_star1 = @(x)((1-rho1^2)/(1-a^2)/eta1^2*x./(1-x)); %lambda^*(R^2),total variance
lambda1_share = sqrt(lambda_star1(R2_1)*share1); %Nxr1 vector, sqrt for std dev
lambda_0_bar = randn(N,r1);
size(lambda_0_bar)
size(lambda1_share)
lambda1 = lambda_0_bar.*lambda1_share; %initial loadings



%% generate observed panel data
C = lambda1*F1';
X=C+e;

stx = repmat(std(X,[],2),1,T);
X=X./stx;
C=C./stx;

