function [Tuning, fpara0] = tuningpara(X,N,T1,T2,k,cons,groups,options,version)
% version
% 0: new lambda
% 1: old program but new lam
% 2: old program but new penalty
% 3: old program

%This program computes the adaptive tuning parameter
%8/20/13, taken from the program model_sel.m

[F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1,T2,k); %principle component estimator of factors and loadings
X1=X(:,1:T1); X2=X(:,T1+1:end);
R1=X1-lambda1_til*F1_til'; R2=X2-lambda2_til*F2_til'; %preliminary estimator of residuals

%initial tuning parameter
e1 = norm(R1,'fro')/(N*T1)^.5+norm(R2,'fro')/(N*T2)^.5;
e2 = norm(R2,'fro')/(N*T2)^.5;

dp = 2;
Ftuning10 = 1/cons*N^(-.5)*max((T1)^(-(dp+1)/2),N^(-(dp+1)/2)); %a constant in the tuning parameter for penalization
Ftuning20 = 1/cons*N^(-.5)*max((T2)^(-(dp+1)/2),N^(-(dp+1)/2)); %a constant in the tuning parameter for penalization

norml     = sqrt(diag(lambda1_til'*lambda1_til))'; %column norms of Lambda1
gamma_til = lambda2_til-lambda1_til;
normg     = sqrt(diag(gamma_til'*gamma_til))'; %column norms of Gamma
Ftuning1  = e1*Ftuning10*N^dp*(norml.^(-2*dp));
Ftuning2  = e2*Ftuning20*N^dp*(normg.^(-2*dp));

%shrinkage estimation based on initial tuning parameter
funObj = @(para)pel_ls(para,X,F1_til,F2_til);  %criterion function function
fpara  = [lambda1_til gamma_til];
fpara0 = reshape(fpara,2*N*k,1); %starting values
Tuning = [Ftuning1';Ftuning2'];

paraest0 = L1GeneralGroup_Auxiliary(funObj,fpara0,Tuning,groups,options);
fest0    = reshape(paraest0,N,2*k); %preliminary estimator

A = diag(fest0'*fest0); A1=A(1:k); A2=A(k+1:2*k);
if norm(A1)==0
    er1 = 0;
else
    er1 = find(A1,1,'last');
end
if norm(A2)==0
    er2 = er1;
else
    er2 = find(A2,1,'last');
    er2 = max(er1,er2);
end

% ------------------------------------------------------------------------
% version control
switch version
    case 0 % new
        %update penalty based on preliminary estimator
        F1_se = F1_til(:,1:er1);
        F2_se = F2_til(:,1:er2);
        lambda1_se = lambda1_til(:,1:er1);
        lambda2_se = lambda2_til(:,1:er2);
        R1_se = X1-lambda1_se*F1_se'; R2_se = X2-lambda2_se*F2_se';
        e1 = norm(R1_se,'fro')/(N*T1)^.5+norm(R2_se,'fro')/(N*T2)^.5;
        e2 = norm(R2_se,'fro')/(N*T2)^.5;
        
        %new on the rotation method
        if er1==er2
            lambda2_tr = rotation_lamb2(lambda1_se,lambda2_se,er1);
        else
            lambda2_tr=lambda2_se;
        end
        
        
        gamma_se=lambda2_tr-[lambda1_se,zeros(N,er2-er1)];
        normg_se = sqrt(diag(gamma_se'*gamma_se))'; %column norms of Gamma_se
        normg_n = normg;
        normg_n(1:er2)=normg_se; %update norm
        
        Ftuning1=e1*Ftuning10*(N^dp)*(norml.^(-2*dp));
        Ftuning2=e2*Ftuning20*(N^dp)*(normg_n.^(-2*dp));
        Tuning=[Ftuning1';Ftuning2'];
        
    case 1 % old but new lambda
        %update penalty based on preliminary estimator
        F1_se = F1_til(:,1:er2);
        F2_se = F2_til(:,1:er2);
        lambda1_se = lambda1_til(:,1:er2);
        lambda2_se = lambda2_til(:,1:er2);
        R1_se = X1-lambda1_se*F1_se'; R2_se = X2-lambda2_se*F2_se';
        e1 = norm(R1_se,'fro')/(N*T1)^.5+norm(R2_se,'fro')/(N*T2)^.5;
        e2 = norm(R2_se,'fro')/(N*T2)^.5;
        
        %new on the rotation method
        if er1==er2
            lambda2_tr = rotation_lamb2(lambda1_se,lambda2_se,er1);
        else
            lambda2_tr=lambda2_se;
        end
        
        gamma_se        = lambda2_tr-lambda1_se; %old way
%         gamma_se=lambda2_tr-[lambda1_se,zeros(N,er2-er1)]; %new way
        normg_se = sqrt(diag(gamma_se'*gamma_se))'; %column norms of Gamma_se
        normg_n = normg;
        normg_n(1:er2)=normg_se; %update norm
        
        Ftuning1=e1*Ftuning10*(N^dp)*(norml.^(-2*dp));
        Ftuning2=e2*Ftuning20*(N^dp)*(normg_n.^(-2*dp));
        Tuning=[Ftuning1';Ftuning2'];
        
    case 2 % old program but new penaty (update r1 and r2)
        %update penalty based on preliminary estimator
        F1_se = F1_til(:,1:er1);
        F2_se = F2_til(:,1:er2);
        lambda1_se = lambda1_til(:,1:er1);
        lambda2_se = lambda2_til(:,1:er2);
        R1_se = X1-lambda1_se*F1_se'; R2_se = X2-lambda2_se*F2_se';
        e1 = norm(R1_se,'fro')/(N*T1)^.5+norm(R2_se,'fro')/(N*T2)^.5;
        e2 = norm(R2_se,'fro')/(N*T2)^.5;
        
%         %new on the rotation method
%         if er1==er2
%             lambda2_tr = rotation_lamb2(lambda1_se,lambda2_se,er1);
%         else
%             lambda2_tr=lambda2_se;
%         end
        [F2_tr,lambda2_tr] = rotation_lamb2_old(F2_se,lambda1_se,lambda2_se,er1); %rotate lambda2_se to update norm of gamma
        
        gamma_se=lambda2_tr-[lambda1_se,zeros(N,er2-er1)];
        normg_se = sqrt(diag(gamma_se'*gamma_se))'; %column norms of Gamma_se
        normg_n = normg;
        normg_n(1:er2)=normg_se; %update norm
        
        Ftuning1=e1*Ftuning10*(N^dp)*(norml.^(-2*dp));
        Ftuning2=e2*Ftuning20*(N^dp)*(normg_n.^(-2*dp));
        Tuning=[Ftuning1';Ftuning2'];
        
    case 3 % completely old program
        %update penalty based on preliminary estimator
        F1_se = F1_til(:,1:er2);
        F2_se = F2_til(:,1:er2);
        lambda1_se = lambda1_til(:,1:er2);
        lambda2_se = lambda2_til(:,1:er2);
        R1_se = X1-lambda1_se*F1_se'; R2_se = X2-lambda2_se*F2_se';
        e1 = norm(R1_se,'fro')/(N*T1)^.5+norm(R2_se,'fro')/(N*T2)^.5;
        e2 = norm(R2_se,'fro')/(N*T2)^.5;
        
%         %new on the rotation method
%         if er1==er2
%             lambda2_tr = rotation_lamb2(lambda1_se,lambda2_se,er1);
%         else
%             lambda2_tr=lambda2_se;
%         end
        
        [F2_tr,lambda2_tr] = rotation_lamb2_old(F2_se,lambda1_se,lambda2_se,er1); %rotate lambda2_se to update norm of gamma
        gamma_se        = lambda2_tr-lambda1_se; %old way
        
%        [F2_tr,lambda2_tr] = rotation_lamb2(F2_se,lambda1_se,lambda2_se,er1); %rotate lambda2_se to update norm of gamma
%         gamma_se=lambda2_tr-[lambda1_se,zeros(N,er2-er1)];
        
        normg_se = sqrt(diag(gamma_se'*gamma_se))'; %column norms of Gamma_se
        normg_n = normg;
        normg_n(1:er2)=normg_se; %update norm
        
        Ftuning1=e1*Ftuning10*(N^dp)*(norml.^(-2*dp));
        Ftuning2=e2*Ftuning20*(N^dp)*(normg_n.^(-2*dp));
        Tuning=[Ftuning1';Ftuning2'];
end