function [TuningV,fpara0V,normlV,normgV] = tuningpara1(X,N,T,k,taov,cons)
%XC. averaging penalty based on unrestricted least squares

%% First-Step Tuning Parameter 
ntao     = size(taov,2);
TuningV  = zeros(2*k,ntao); %collecting tuning vector for searching range
fpara0V = zeros(2*N*k,ntao); %set-up starting value
normlV = zeros(k,ntao);
normgV = zeros(k,ntao);

for j = 1:ntao
    
tao = taov(j);
T1  = floor(T*tao); T2=T-T1;      
    
[F1_til,F2_til,lambda1_til,lambda2_til] = pc(X,T1,T2,k); %principle component estimator of factors and loadings
X1=X(:,1:T1); X2=X(:,T1+1:end);
R1=X1-lambda1_til*F1_til'; R2=X2-lambda2_til*F2_til'; %preliminary estimator of residuals

%initial tuning parameter
e1 = norm(R1,'fro')/(N*T1)^.5+norm(R2,'fro')/(N*T2)^.5;
e2 = norm(R2,'fro')/(N*T2)^.5;

dp = 2; 

Ftuning10 = 1/cons*N^(-.5)*max((T1)^(-(dp+1)/2),N^(-(dp+1)/2)); %a constant in the tuning parameter for penalization
Ftuning20 = 1/cons*N^(-.5)*max((T2)^(-(dp+1)/2),N^(-(dp+1)/2)); %a constant in the tuning parameter for penalization

norml     = sqrt(diag(lambda1_til'*lambda1_til)); %column norms of Lambda1
gamma_til = lambda2_til-lambda1_til;
normg     = sqrt(diag(gamma_til'*gamma_til)); %column norms of Gamma

norml2 = sqrt(diag(lambda2_til'*lambda2_til));
normg = min(normg,norml2);

Ftuning1  = e1*Ftuning10*N^dp*(norml.^(-2*dp));
Ftuning2  = e2*Ftuning20*N^dp*(normg.^(-2*dp));

fpara  = [lambda1_til gamma_til];
fpara0 = reshape(fpara,2*N*k,1); %starting values
Tuning = [Ftuning1;Ftuning2];

TuningV(:,j)  = Tuning; 
fpara0V(:,j) = fpara0;
normlV(:,j)=norml;
normgV(:,j)=normg;

end

